@@ -728,20 +728,11 @@ static void _add_module_to_grid (CairoDockCategoryWidgetTable *pCategoryWidget,
 		pCategoryWidget->iNbItemsInCurrentRow = 0;
 		pCategoryWidget->iNbRows ++;
 	}
-	#if GTK_CHECK_VERSION (3, 4, 0)
 	gtk_grid_attach (GTK_GRID (pCategoryWidget->pTable),
 		pWidget,
 		pCategoryWidget->iNbItemsInCurrentRow+1,
 		pCategoryWidget->iNbRows+1,
 		1, 1);
-	#else
-	gtk_table_attach_defaults (GTK_TABLE (pCategoryWidget->pTable),
-		pWidget,
-		pCategoryWidget->iNbItemsInCurrentRow,
-		pCategoryWidget->iNbItemsInCurrentRow+1,
-		pCategoryWidget->iNbRows,
-		pCategoryWidget->iNbRows+1);
-	#endif
 	pCategoryWidget->iNbItemsInCurrentRow ++;
 }
 
@@ -784,17 +775,11 @@ static void on_click_toggle_activated (GtkButton *button, G_GNUC_UNUSED gpointer
 		pCategoryWidget->iNbItemsInCurrentRow = 0;
 		
 		gtk_widget_destroy(pCategoryWidget->pTable);
-		#if GTK_CHECK_VERSION (3, 4, 0)
 		pCategoryWidget->pTable = gtk_grid_new ();
 		gtk_grid_set_row_spacing (GTK_GRID (pCategoryWidget->pTable), CAIRO_DOCK_FRAME_MARGIN);
 		gtk_grid_set_row_homogeneous (GTK_GRID (pCategoryWidget->pTable), TRUE);
 		gtk_grid_set_column_spacing (GTK_GRID (pCategoryWidget->pTable), CAIRO_DOCK_FRAME_MARGIN);
 		gtk_grid_set_column_homogeneous (GTK_GRID (pCategoryWidget->pTable), TRUE);
-		#else
-		pCategoryWidget->pTable = gtk_table_new (1, s_iNbButtonsByRow, TRUE);
-		gtk_table_set_row_spacings (GTK_TABLE (pCategoryWidget->pTable), CAIRO_DOCK_FRAME_MARGIN);
-		gtk_table_set_col_spacings (GTK_TABLE (pCategoryWidget->pTable), CAIRO_DOCK_FRAME_MARGIN);
-		#endif
 		gtk_container_add (GTK_CONTAINER (pCategoryWidget->pFrame), pCategoryWidget->pTable);
 	}
 
@@ -892,9 +877,10 @@ static void on_click_back_button (G_GNUC_UNUSED GtkButton *button, G_GNUC_UNUSED
 	_show_group_or_category (pPrevPlace);
 }
 
-static void _on_group_dialog_destroyed (G_GNUC_UNUSED gpointer data)
+static gboolean _on_group_dialog_destroyed (G_GNUC_UNUSED gpointer data)
 {
 	s_pDialog = NULL;
+	return GLDI_NOTIFICATION_LET_PASS;
 }
 static gboolean _show_group_dialog (CairoDockGroupDescription *pGroupDescription)
 {
@@ -903,29 +889,14 @@ static gboolean _show_group_dialog (CairoDockGroupDescription *pGroupDescription
 	GtkWidget *pPreviewImage = s_pPreviewImage;
 	if (pGroupDescription->cPreviewFilePath != NULL && strcmp (pGroupDescription->cPreviewFilePath, "none") != 0)
 	{
+		int scale = 1;
+		GdkWindow* gdkwindow = gldi_container_get_gdk_window (CAIRO_CONTAINER (g_pMainDock));
+		if (gdkwindow) scale = gdk_window_get_scale_factor (gdkwindow);
+		
 		//g_print ("on recupere la prevue de %s\n", pGroupDescription->cPreviewFilePath);
-		int iPreviewWidth, iPreviewHeight;
-		GdkPixbuf *pPreviewPixbuf = NULL;
-		if (gdk_pixbuf_get_file_info (pGroupDescription->cPreviewFilePath, &iPreviewWidth, &iPreviewHeight) != NULL)
-		{
-			if (iPreviewWidth > CAIRO_DOCK_PREVIEW_WIDTH)
-			{
-				iPreviewHeight *= (double)CAIRO_DOCK_PREVIEW_WIDTH/iPreviewWidth;
-				iPreviewWidth = CAIRO_DOCK_PREVIEW_WIDTH;
-			}
-			if (iPreviewHeight > CAIRO_DOCK_PREVIEW_HEIGHT)
-			{
-				iPreviewWidth *= (double)CAIRO_DOCK_PREVIEW_HEIGHT/iPreviewHeight;
-				iPreviewHeight = CAIRO_DOCK_PREVIEW_HEIGHT;
-			}
-			if (iPreviewWidth > iPreviewWidgetWidth)
-			{
-				iPreviewHeight *= (double)iPreviewWidgetWidth/iPreviewWidth;
-				iPreviewWidth = iPreviewWidgetWidth;
-			}
-			//g_print ("preview : %dx%d\n", iPreviewWidth, iPreviewHeight);
-			pPreviewPixbuf = gdk_pixbuf_new_from_file_at_size (pGroupDescription->cPreviewFilePath, iPreviewWidth, iPreviewHeight, NULL);
-		}
+		int iTargetWidth = (iPreviewWidgetWidth < CAIRO_DOCK_PREVIEW_WIDTH) ? iPreviewWidgetWidth : CAIRO_DOCK_PREVIEW_WIDTH;
+		GdkPixbuf *pPreviewPixbuf = cairo_dock_load_gdk_pixbuf_with_max_size (pGroupDescription->cPreviewFilePath,
+			iTargetWidth * scale, CAIRO_DOCK_PREVIEW_HEIGHT * scale);
 		if (pPreviewPixbuf == NULL)
 		{
 			cd_warning ("no preview available");
@@ -938,17 +909,33 @@ static gboolean _show_group_dialog (CairoDockGroupDescription *pGroupDescription
 		else 
 			gtk_widget_show (s_pPreviewBox);
 
-		gtk_image_set_from_pixbuf (GTK_IMAGE (pPreviewImage), pPreviewPixbuf);
+		cairo_surface_t *surface = gdk_cairo_surface_create_from_pixbuf (pPreviewPixbuf, scale, NULL);
+		gtk_image_set_from_surface (GTK_IMAGE (pPreviewImage), surface);
+		cairo_surface_destroy (surface);
 		g_object_unref (pPreviewPixbuf);
 	}
 	
 	// show the module's description
 	gldi_object_unref (GLDI_OBJECT(s_pDialog));
+	s_pDialog = NULL;
+	
+	// if we are running on Wayland and our window is not active, the dialog could get hidden soon
+	// (in on_leave_group_button()), so let's not show it
+	if (gldi_container_is_wayland_backend () && !gtk_window_is_active (GTK_WINDOW (s_pMainWindow)))
+		return FALSE;
 	
 	Icon *pIcon = cairo_dock_get_current_active_icon ();  // most probably the appli-icon representing the config window.
 	if (pIcon == NULL || cairo_dock_get_icon_container(pIcon) == NULL || cairo_dock_icon_is_being_removed (pIcon))
 		pIcon = gldi_icons_get_any_without_dialog ();
 	GldiContainer *pContainer = (pIcon != NULL ? cairo_dock_get_icon_container (pIcon) : NULL);
+	if (pContainer && GLDI_OBJECT_IS_DOCK (pContainer))
+	{
+		// ensure that this is not a subdock (which will likely be hidden)
+		CairoDock *pDock = CAIRO_DOCK (pContainer);
+		while (pDock->iRefCount > 0)
+			pIcon = cairo_dock_search_icon_pointing_on_dock (pDock, &pDock);
+		pContainer = CAIRO_CONTAINER (pDock);
+	}
 	
 	CairoDialogAttr attr;
 	memset (&attr, 0, sizeof (CairoDialogAttr));
@@ -998,6 +985,7 @@ static gboolean on_enter_group_button (GtkButton *button, G_GNUC_UNUSED GdkEvent
 static gboolean _check_group_button (G_GNUC_UNUSED gpointer data)
 {
 	GldiWindowActor *pActiveWindow = gldi_windows_get_active ();
+	// this function is only called on X11
 	Window Xid = GDK_WINDOW_XID (gtk_widget_get_window (s_pMainWindow));
 	if (gldi_window_get_id (pActiveWindow) != Xid)  // we're not the active window any more, so the 'leave' event was probably due to an Alt+Tab -> the mouse is really out of the button.
 	{
@@ -1029,22 +1017,25 @@ static gboolean on_leave_group_button (GtkButton *button, GdkEventCrossing *pEve
 	// check that we are really outside of the button (this may be false if the dialog is appearing under the mouse and has not yet its input shape (X lag)).
 	if (pEvent->detail != GDK_NOTIFY_ANCESTOR)  // a LeaveNotify event not within the same window (ie, either an Alt+Tab or the dialog that spawned under the cursor)
 	{
-		GtkAllocation allocation;
-		gtk_widget_get_allocation (GTK_WIDGET (button), &allocation);
-		int x, y;
-		#if GTK_CHECK_VERSION (3, 4, 0)
-		GdkDevice *pDevice = gdk_device_manager_get_client_pointer (
-			gdk_display_get_device_manager (gtk_widget_get_display (GTK_WIDGET (button))));
-		gdk_window_get_device_position (gtk_widget_get_window (GTK_WIDGET (button)), pDevice, &x, &y, NULL);
-		x -= allocation.x;
-		y -= allocation.y;
-		#else
-		gtk_widget_get_pointer (GTK_WIDGET (button), &x, &y);
-		#endif
-		if (x >= 0 && x < allocation.width && y >= 0 && y < allocation.height)  // we are actually still inside the button, ignore the event, we'll get an 'enter' event as soon as the dialog's input shape is ready.
+		if (gldi_container_is_wayland_backend ())
 		{
-			s_iSidCheckGroupButton = g_timeout_add (1000, _check_group_button, NULL);  // check in a moment if we left the button because of the dialog or because of another window (alt+tab).
-			return FALSE;
+			if (gtk_window_is_active (GTK_WINDOW (s_pMainWindow))) return FALSE;
+		}
+		else
+		{
+			GtkAllocation allocation;
+			gtk_widget_get_allocation (GTK_WIDGET (button), &allocation);
+			int x, y;
+			GdkSeat *pSeat = gdk_display_get_default_seat (gtk_widget_get_display (GTK_WIDGET (button)));
+			GdkDevice *pDevice = gdk_seat_get_pointer (pSeat);
+			gdk_window_get_device_position (gtk_widget_get_window (GTK_WIDGET (button)), pDevice, &x, &y, NULL);
+			x -= allocation.x;
+			y -= allocation.y;
+			if (x >= 0 && x < allocation.width && y >= 0 && y < allocation.height)  // we are actually still inside the button, ignore the event, we'll get an 'enter' event as soon as the dialog's input shape is ready.
+			{
+				s_iSidCheckGroupButton = g_timeout_add (1000, _check_group_button, NULL);  // check in a moment if we left the button because of the dialog or because of another window (alt+tab).
+				return FALSE;
+			}
 		}
 	}
 	
@@ -1089,8 +1080,7 @@ static void cairo_dock_free_categories (void)
 {
 	memset (s_pCategoryWidgetTables, 0, sizeof (s_pCategoryWidgetTables));  // les widgets a l'interieur sont detruits avec la fenetre.
 	
-	g_list_foreach (s_pGroupDescriptionList, (GFunc)_cairo_dock_free_group_description, NULL);
-	g_list_free (s_pGroupDescriptionList);
+	g_list_free_full (s_pGroupDescriptionList, (GDestroyNotify)_cairo_dock_free_group_description);
 	s_pGroupDescriptionList = NULL;
 	s_pCurrentGroup = NULL;
 	
@@ -1321,7 +1311,7 @@ static GtkToolItem *_make_toolbutton (const gchar *cLabel, const gchar *cImage,
 		gtk_tool_button_set_label (GTK_TOOL_BUTTON (pWidget), cLabel);
 		return pWidget;
 	}
-	GtkWidget *pImage = _gtk_image_new_from_file (cImage, iSize);
+	GtkWidget *pImage = cairo_dock_gui_image_from_file (cImage, iSize);
 	GtkToolItem *pWidget = gtk_toggle_tool_button_new ();
 	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (pWidget), pImage);
 	if (cLabel == NULL)
@@ -1332,10 +1322,10 @@ static GtkToolItem *_make_toolbutton (const gchar *cLabel, const gchar *cImage,
 	gtk_label_set_markup (GTK_LABEL (pLabel), cLabel2);
 	g_free (cLabel2);
 	
-	GtkWidget *pAlign = gtk_alignment_new (0., 0.5, 0., 1.);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (pAlign), 0, 0, CAIRO_DOCK_FRAME_MARGIN, 0);
-	gtk_container_add (GTK_CONTAINER (pAlign), pLabel);
-	gtk_tool_button_set_label_widget (GTK_TOOL_BUTTON (pWidget), pAlign);
+	GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_widget_set_margin_start (box, CAIRO_DOCK_FRAME_MARGIN);
+	gtk_box_pack_start (GTK_BOX (box), pLabel, FALSE, FALSE, 0);
+	gtk_tool_button_set_label_widget (GTK_TOOL_BUTTON (pWidget), box);
 	
 	return pWidget;
 }
@@ -1386,7 +1376,7 @@ static CairoDockGroupDescription *_add_group_button (const gchar *cGroupName, co
 	g_signal_connect (G_OBJECT (pGroupButton), "leave-notify-event", G_CALLBACK(on_leave_group_button), NULL);
 
 	GtkWidget *pButtonHBox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, CAIRO_DOCK_FRAME_MARGIN);
-	GtkWidget *pImage = _gtk_image_new_from_file (pGroupDescription->cIcon, GTK_ICON_SIZE_LARGE_TOOLBAR);
+	GtkWidget *pImage = cairo_dock_gui_image_from_file (pGroupDescription->cIcon, GTK_ICON_SIZE_LARGE_TOOLBAR);
 	gtk_box_pack_start (GTK_BOX (pButtonHBox), pImage, FALSE, FALSE, 0);
 	pGroupDescription->pLabel = gtk_label_new (pGroupDescription->cTitle);
 	gtk_box_pack_start (GTK_BOX (pButtonHBox),
@@ -1628,9 +1618,12 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath)  // 'cC
 	}
 	s_pMainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	//gtk_container_set_border_width (s_pMainWindow, CAIRO_DOCK_FRAME_MARGIN);
-	gchar *cIconPath = g_strdup_printf ("%s/%s", CAIRO_DOCK_SHARE_DATA_DIR, CAIRO_DOCK_ICON);
-	gtk_window_set_icon_from_file (GTK_WINDOW (s_pMainWindow), cIconPath, NULL);
-	g_free (cIconPath);
+	GdkPixbuf *pIconPixbuf = cairo_dock_load_gdk_pixbuf (GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_ICON, -1, -1);
+	if (pIconPixbuf)
+	{
+		gtk_window_set_icon (GTK_WINDOW (s_pMainWindow), pIconPixbuf);
+		g_object_unref (pIconPixbuf);
+	}
 
 	GtkWidget *pMainHBox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_container_add (GTK_CONTAINER (s_pMainWindow), pMainHBox);
