@@ -25,14 +25,10 @@
 #define __USE_POSIX
 #include <time.h>
 #include <glib/gstdio.h>
-#include <glib/gi18n.h>
-#include <gdk/gdkx.h>
 
 #include "config.h"
-#include "cairo-dock-config.h"
 #include "cairo-dock-keyfile-utilities.h"
-#include "cairo-dock-module-manager.h"
-#include "cairo-dock-module-factory.h"
+#include "cairo-dock-desklet-factory.h"  // cairo_dock_desklet_is_sticky
 #include "cairo-dock-dock-manager.h"
 #include "cairo-dock-gui-manager.h"
 #include "cairo-dock-gui-factory.h"
@@ -40,17 +36,15 @@
 #include "cairo-dock-log.h"
 #include "cairo-dock-packages.h"
 #include "cairo-dock-keybinder.h"
-#include "cairo-dock-X-manager.h"
-#include "cairo-dock-X-utilities.h"
+#include "cairo-dock-desktop-manager.h"  // gldi_desktop_get_width
 #include "cairo-dock-themes-manager.h"
 #include "cairo-dock-dialog-manager.h"
 #include "cairo-dock-gui-backend.h"
 #include "cairo-dock-gui-commons.h"
 #include "cairo-dock-icon-manager.h" // cairo_dock_search_icon_s_path
 
-#define CAIRO_DOCK_PLUGINS_EXTRAS_URL "http://extras.glx-dock.org"
+#define CAIRO_DOCK_PLUGINS_EXTRAS_URL "https://github.com/Cairo-Dock/cairo-dock-plug-ins-extras" // "http://extras.glx-dock.org"
 
-extern CairoDockDesktopGeometry g_desktopGeometry;
 extern gchar *g_cCairoDockDataDir;
 
 
@@ -85,8 +79,8 @@ void cairo_dock_update_desklet_widgets (CairoDesklet *pDesklet, GSList *pWidgetL
 			0, 0, NULL, _cairo_dock_set_value_in_pair, NULL);
 	}
 	
-	int iRelativePositionX = (pDesklet->container.iWindowPositionX + pDesklet->container.iWidth/2 <= gldi_get_desktop_width()/2 ? pDesklet->container.iWindowPositionX : pDesklet->container.iWindowPositionX - gldi_get_desktop_width());
-	int iRelativePositionY = (pDesklet->container.iWindowPositionY + pDesklet->container.iHeight/2 <= gldi_get_desktop_height()/2 ? pDesklet->container.iWindowPositionY : pDesklet->container.iWindowPositionY - gldi_get_desktop_height());
+	int iRelativePositionX = (pDesklet->container.iWindowPositionX + pDesklet->container.iWidth/2 <= gldi_desktop_get_width()/2 ? pDesklet->container.iWindowPositionX : pDesklet->container.iWindowPositionX - gldi_desktop_get_width());
+	int iRelativePositionY = (pDesklet->container.iWindowPositionY + pDesklet->container.iHeight/2 <= gldi_desktop_get_height()/2 ? pDesklet->container.iWindowPositionY : pDesklet->container.iWindowPositionY - gldi_desktop_get_height());
 	
 	pGroupKeyWidget = cairo_dock_gui_find_group_key_widget_in_list (pWidgetList, "Desklet", "x position");
 	g_return_if_fail (pGroupKeyWidget != NULL && pGroupKeyWidget->pSubWidgetList != NULL);
@@ -108,7 +102,7 @@ void cairo_dock_update_desklet_visibility_widgets (CairoDesklet *pDesklet, GSLis
 {
 	CairoDockGroupKeyWidget *pGroupKeyWidget;
 	GtkWidget *pOneWidget;
-	gboolean bIsSticky = cairo_dock_desklet_is_sticky (pDesklet);
+	gboolean bIsSticky = gldi_desklet_is_sticky (pDesklet);
 	CairoDeskletVisibility iVisibility = pDesklet->iVisibility;
 	
 	pGroupKeyWidget = cairo_dock_gui_find_group_key_widget_in_list (pWidgetList, "Desklet", "accessibility");
@@ -138,60 +132,11 @@ void cairo_dock_update_is_detached_widget (gboolean bIsDetached, GSList *pWidget
 }
 
 
-#define DISTANT_DIR "3.2.0"
+// #define DISTANT_DIR "3.4.0"
 gchar *cairo_dock_get_third_party_applets_link (void)
 {
-	return g_strdup_printf (CAIRO_DOCK_PLUGINS_EXTRAS_URL"/"DISTANT_DIR);
-}
-
-
-/**
- * Return absolute position of the bottom left corner of a widget to place a menu.
- * use with @see cairo_dock_place_menu_at_position
- * see @see cairo_dock_popup_menu_under_widget for an example
- */
-GtkRequisition *cairo_dock_get_widget_bottom_position (GtkWidget *pWidget)
-{
-	GtkRequisition *pPosition = g_new (GtkRequisition, 1);
-	gint dx = 0, dy = 0;
-	GtkRequisition req;
-	
-	// Top left drawable window coordinates on screen. (inside decorations)
-	// We need to get the toplevel widget origin. 
-	// Otherwise, it would give wrong results if called on the widget in a scrolled area.
-	// (drawable surface origin can even be out of the screen).
-	GtkWidget *pTopLevel = gtk_widget_get_toplevel (pWidget);
-	gdk_window_get_origin (gtk_widget_get_window (pTopLevel), &pPosition->width, &pPosition->height);
-	
-	// Widget position inside the window.
-	gtk_widget_translate_coordinates (pWidget, pTopLevel, 0, 0, &dx, &dy);
-	
-	// Widget height.
-	#if (GTK_MAJOR_VERSION < 3)
-		gtk_widget_size_request (pWidget, &req);
-	#else
-		gtk_widget_get_preferred_size (pWidget, &req, NULL);
-	#endif
-	
-	pPosition->width += dx;
-	pPosition->height += dy + req.height;
-
-	return pPosition;
-}
-
-
-/**
- * Convenient GtkMenuPositionFunc callback function to position menu.
- * use with @see cairo_dock_get_widget_bottom_position
- */
-void cairo_dock_place_menu_at_position (G_GNUC_UNUSED GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkRequisition *pPosition)
-{
-	*push_in = TRUE;
-	if (pPosition != NULL)
-	{
-		*x = pPosition->width;
-		*y = pPosition->height;
-	}
+	// return g_strdup_printf (CAIRO_DOCK_PLUGINS_EXTRAS_URL"/"DISTANT_DIR);
+	return g_strdup_printf (CAIRO_DOCK_PLUGINS_EXTRAS_URL);
 }
 
 
@@ -200,18 +145,8 @@ void cairo_dock_place_menu_at_position (G_GNUC_UNUSED GtkMenu *menu, gint *x, gi
  */
 void cairo_dock_popup_menu_under_widget (GtkWidget *pWidget, GtkMenu *pMenu)
 {
-	GtkRequisition *pPosition = pWidget == NULL ? NULL : cairo_dock_get_widget_bottom_position (pWidget);
-	
-	gtk_menu_popup (pMenu,
-		NULL,
-		NULL,
-		(GtkMenuPositionFunc) cairo_dock_place_menu_at_position,
-		pPosition,
-		0, // pEventButton->button
-		gtk_get_current_event_time ()); // pEventButton->time
-	
-	if (pPosition != NULL)
-		g_free (pPosition);
+	if (pWidget) gtk_menu_popup_at_widget (pMenu, pWidget, GDK_GRAVITY_SOUTH_WEST, GDK_GRAVITY_NORTH_WEST, NULL);
+	else gtk_menu_popup_at_pointer (pMenu, NULL);
 }
 
 /**
@@ -235,10 +170,6 @@ gchar * cairo_dock_get_icon_for_gui (const gchar *cGroupName, const gchar *cIcon
 				cIconPath = g_strdup (cIcon);
 			}
 		}
-		else if (strncmp (cIcon, "gtk-", 4) == 0)
-		{
-			cIconPath = g_strdup (cIcon);
-		}
 		else  // categorie ou module interne.
 		{
 			if (! bFastLoad)
@@ -258,7 +189,7 @@ gchar * cairo_dock_get_icon_for_gui (const gchar *cGroupName, const gchar *cIcon
 			if (cIconPath == NULL)
 			{
 				cIconPath = cairo_dock_search_icon_s_path (cIcon, iSize);
-				if (cIconPath == NULL && cShareDataDir) // maybe we don't have any icon with this name
+				if (cIconPath == NULL) // maybe we don't have any icon with this name
 					cIconPath = cShareDataDir ? g_strdup_printf ("%s/icon", cShareDataDir) : g_strdup (cIcon);
 			}
 		}
