@@ -181,7 +181,12 @@ GtkWidget *cairo_dock_build_generic_gui_window2 (const gchar *cTitle, int iWidth
 {
 	//\_____________ make a new window.
 	GtkWidget *pMainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_icon_from_file (GTK_WINDOW (pMainWindow), GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_ICON, NULL);
+	GdkPixbuf *pIconPixbuf = cairo_dock_load_gdk_pixbuf (GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_ICON, -1, -1);
+	if (pIconPixbuf)
+	{
+		gtk_window_set_icon (GTK_WINDOW (pMainWindow), pIconPixbuf);
+		g_object_unref (pIconPixbuf);
+	}
 	if (cTitle != NULL)
 		gtk_window_set_title (GTK_WINDOW (pMainWindow), cTitle);
 	
@@ -281,22 +286,18 @@ static GtkWidget *_make_notebook_label (const gchar *cLabel, const gchar *cImage
 {
 	GtkWidget *hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, CAIRO_DOCK_FRAME_MARGIN);
 	
-	GtkWidget *pImage = _gtk_image_new_from_file (cImage, iSize);
-	GtkWidget *pAlign = gtk_alignment_new (0.5, 0.5, 0., 1.);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (pAlign), 0, 0, CAIRO_DOCK_FRAME_MARGIN, 0);
-	gtk_container_add (GTK_CONTAINER (pAlign), pImage);
+	GtkWidget *pImage = cairo_dock_gui_image_from_file (cImage, iSize);
+	gtk_widget_set_margin_start (pImage, CAIRO_DOCK_FRAME_MARGIN);
 	gtk_box_pack_start (GTK_BOX (hbox),
-		pAlign,
+		pImage,
 		FALSE,
 		FALSE,
 		0);
 	
 	GtkWidget *pLabel = gtk_label_new (cLabel);
-	pAlign = gtk_alignment_new (0.5, 0.5, 0., 1.);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (pAlign), 0, 0, CAIRO_DOCK_FRAME_MARGIN, 0);
-	gtk_container_add (GTK_CONTAINER (pAlign), pLabel);
+	gtk_widget_set_margin_start (pLabel, CAIRO_DOCK_FRAME_MARGIN);
 	gtk_box_pack_start (GTK_BOX (hbox),
-		pAlign,
+		pLabel,
 		FALSE,
 		FALSE,
 		0);	
@@ -408,6 +409,7 @@ static void cairo_dock_select_category (GtkWidget *pMainWindow, CDCategoryEnum i
 {
 	GtkNotebook *pNoteBook = g_object_get_data (G_OBJECT (pMainWindow), "notebook");
 	gtk_notebook_set_current_page (pNoteBook, iCategory);  // will first emit a 'switch-page' signal, which will build the widget if necessary.
+	g_signal_emit_by_name (pNoteBook, "switch-page", gtk_notebook_get_nth_page(pNoteBook, iCategory), iCategory, NULL);  // see the comment on the previous line ? well, it's not true anymore...
 }
 
 
