@@ -178,23 +178,10 @@ static void _show_all_windows (GList *pIcons)
 
 static gboolean _launch_icon_command (Icon *icon)
 {
-	if (icon->cCommand == NULL)
-		return GLDI_NOTIFICATION_LET_PASS;
+	if (gldi_class_is_starting (icon->cClass) || gldi_icon_is_launching (icon))  // do not launch it twice (avoid wrong double click); so we can't launch an app several times rapidly (must wait until it's launched), but it's probably not a useful feature anyway
+		return GLDI_NOTIFICATION_INTERCEPT;
 	
-	gboolean bSuccess = FALSE;
-	if (*icon->cCommand == '<')  // shortkey
-	{
-		bSuccess = cairo_dock_trigger_shortkey (icon->cCommand);
-		if (!bSuccess)
-			bSuccess = gldi_icon_launch_command (icon);
-	}
-	else  // normal command
-	{
-		bSuccess = gldi_icon_launch_command (icon);
-		if (! bSuccess)
-			bSuccess = cairo_dock_trigger_shortkey (icon->cCommand);
-	}
-	if (! bSuccess)
+	if (! gldi_icon_launch_command (icon))
 	{
 		gldi_icon_request_animation (icon, "blink", 1);  // 1 blink if fail.
 	}
@@ -224,11 +211,11 @@ gboolean cairo_dock_notification_click_icon (G_GNUC_UNUSED gpointer pUserData, I
 		if (myTaskbarParam.bPresentClassOnClick // if we want to use this feature
 		&& (!myDocksParam.bShowSubDockOnClick  // if sub-docks are shown on mouse over
 			|| gldi_container_is_visible (CAIRO_CONTAINER (icon->pSubDock)))  // or this sub-dock is already visible
-		&& gldi_desktop_present_class (icon->cClass)) // we use the scale plugin if it's possible
+		&& gldi_desktop_present_class (icon->cClass, pContainer)) // we use the scale plugin if it's possible
 		{
 			_show_all_windows (icon->pSubDock->icons); // show all windows
 			// in case the dock is visible or about to be visible, hide it, as it would confuse the user to have both.
-			cairo_dock_emit_leave_signal (CAIRO_CONTAINER (icon->pSubDock));
+			gldi_dock_leave_synthetic (icon->pSubDock);
 			return GLDI_NOTIFICATION_INTERCEPT;
 		}
 	}
@@ -242,7 +229,8 @@ gboolean cairo_dock_notification_click_icon (G_GNUC_UNUSED gpointer pUserData, I
 	else if (CAIRO_DOCK_IS_APPLI (icon) && ! CAIRO_DOCK_IS_APPLET (icon))  // icon holding an appli, but not being an applet -> show/hide the window.
 	{
 		GldiWindowActor *pAppli = icon->pAppli;
-		if (gldi_windows_get_active () == pAppli && myTaskbarParam.bMinimizeOnClick && ! pAppli->bIsHidden && gldi_window_is_on_current_desktop (pAppli))  // ne marche que si le dock est une fenêtre de type 'dock', sinon il prend le focus.
+		if (gldi_windows_get_active () == pAppli && myTaskbarParam.bMinimizeOnClick && ! pAppli->bIsHidden &&
+				(!gldi_window_manager_can_track_workspaces () || gldi_window_is_on_current_desktop (pAppli)))  // ne marche que si le dock est une fenêtre de type 'dock', sinon il prend le focus.
 			gldi_window_minimize (pAppli);
 		else
 			gldi_window_show (pAppli);
@@ -258,8 +246,7 @@ gboolean cairo_dock_notification_click_icon (G_GNUC_UNUSED gpointer pUserData, I
 	}
 	else if (CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (icon))  // finally, launcher being none of the previous cases -> launch the command
 	{
-		if (! gldi_class_is_starting (icon->cClass) && ! gldi_icon_is_launching (icon))  // do not launch it twice (avoid wrong double click) => if we want to launch it 2 times in a row, we have to use Shift + Click
-			_launch_icon_command (icon);
+		_launch_icon_command (icon);
 	}
 	else  // for applets and their sub-icons, let the module-instance handles the click; for separators, no action.
 	{
@@ -275,44 +262,48 @@ gboolean cairo_dock_notification_middle_click_icon (G_GNUC_UNUSED gpointer pUser
 		return GLDI_NOTIFICATION_LET_PASS;
 	CairoDock *pDock = CAIRO_DOCK (pContainer);
 	
-	if (CAIRO_DOCK_IS_APPLI (icon) && ! CAIRO_DOCK_IS_APPLET (icon) && myTaskbarParam.iActionOnMiddleClick != 0)
+	if (CAIRO_DOCK_IS_APPLI (icon) && ! CAIRO_DOCK_IS_APPLET (icon) && myTaskbarParam.iActionOnMiddleClick != CAIRO_APPLI_ACTION_NONE)
 	{
 		switch (myTaskbarParam.iActionOnMiddleClick)
 		{
-			case 1:  // close
+			case CAIRO_APPLI_ACTION_CLOSE:  // close
 				gldi_window_close (icon->pAppli);
 			break;
-			case 2:  // minimise
+			case CAIRO_APPLI_ACTION_MINIMISE:  // minimise
 				if (! icon->pAppli->bIsHidden)
 				{
 					gldi_window_minimize (icon->pAppli);
 				}
 			break;
-			case 3:  // launch new
-				if (icon->cCommand != NULL)
+			case CAIRO_APPLI_ACTION_LAUNCH_NEW:  // launch new
+				if (icon->pAppInfo)
 				{
-					gldi_object_notify (pDock, NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
+					gldi_object_notify (pDock, NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // emulate a shift+left click
 				}
 			break;
+			default:  // nothing
+			break;
 		}
 		return GLDI_NOTIFICATION_INTERCEPT;
 	}
-	else if (CAIRO_DOCK_IS_MULTI_APPLI (icon) && myTaskbarParam.iActionOnMiddleClick != 0)
+	else if (CAIRO_DOCK_IS_MULTI_APPLI (icon) && myTaskbarParam.iActionOnMiddleClick != CAIRO_APPLI_ACTION_NONE)
 	{
 		switch (myTaskbarParam.iActionOnMiddleClick)
 		{
-			case 1:  // close
+			case CAIRO_APPLI_ACTION_CLOSE:  // close
 				_cairo_dock_close_all_in_class_subdock (icon);
 			break;
-			case 2:  // minimise
+			case CAIRO_APPLI_ACTION_MINIMISE:  // minimise
 				_cairo_dock_hide_show_in_class_subdock (icon);
 			break;
-			case 3:  // launch new
-				if (icon->cCommand != NULL)
+			case CAIRO_APPLI_ACTION_LAUNCH_NEW:  // launch new
+				if (icon->pAppInfo)
 				{
-					gldi_object_notify (CAIRO_CONTAINER (pDock), NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
+					gldi_object_notify (CAIRO_CONTAINER (pDock), NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // emulate a shift+left click
 				}
 			break;
+			default:  // nothing
+			break;
 		}
 		
 		return GLDI_NOTIFICATION_INTERCEPT;
@@ -321,9 +312,9 @@ gboolean cairo_dock_notification_middle_click_icon (G_GNUC_UNUSED gpointer pUser
 }
 
 
-gboolean cairo_dock_notification_scroll_icon (G_GNUC_UNUSED gpointer pUserData, Icon *icon, G_GNUC_UNUSED GldiContainer *pContainer, int iDirection)
+gboolean cairo_dock_notification_scroll_icon (G_GNUC_UNUSED gpointer pUserData, Icon *icon, G_GNUC_UNUSED GldiContainer *pContainer, int iDirection, gboolean)
 {
-	if (CAIRO_DOCK_IS_MULTI_APPLI (icon) || CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon))  // on emule un alt+tab sur la liste des applis du sous-dock.
+	if (CAIRO_DOCK_IS_MULTI_APPLI (icon) || CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon))  // emulate an alt+tab on the list of applis of the sub-dock
 	{
 		_cairo_dock_show_prev_next_in_subdock (icon, iDirection == GDK_SCROLL_DOWN);
 	}
@@ -337,67 +328,86 @@ gboolean cairo_dock_notification_scroll_icon (G_GNUC_UNUSED gpointer pUserData,
 }
 
 
-gboolean cairo_dock_notification_drop_data (G_GNUC_UNUSED gpointer pUserData, const gchar *cReceivedData, Icon *icon, double fOrder, GldiContainer *pContainer)
+gboolean cairo_dock_notification_drop_data_selection (G_GNUC_UNUSED gpointer pUserData,
+		GtkSelectionData *selection_data, Icon *icon, double fOrder, GldiContainer *pContainer,
+		gboolean *bHandled)
 {
-	cd_debug ("take the drop");
 	if (! CAIRO_DOCK_IS_DOCK (pContainer))
 		return GLDI_NOTIFICATION_LET_PASS;
-	
 	CairoDock *pDock = CAIRO_DOCK (pContainer);
-	CairoDock *pReceivingDock = pDock;
-	if (g_str_has_suffix (cReceivedData, ".desktop"))  // .desktop -> add a new launcher if dropped on or amongst launchers. 
+	
+	gchar **data = NULL;
+	data = gtk_selection_data_get_uris (selection_data);
+	if (data) cd_debug ("got URIs");
+	else
 	{
-		cd_debug (" dropped a .desktop");
-		if (! myTaskbarParam.bMixLauncherAppli && CAIRO_DOCK_ICON_TYPE_IS_APPLI (icon))
-			return GLDI_NOTIFICATION_LET_PASS;
-		cd_debug (" add it");
-		if (fOrder == CAIRO_DOCK_LAST_ORDER && CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon) && icon->pSubDock != NULL)  // drop onto a container icon.
+		guchar *tmp = gtk_selection_data_get_text (selection_data);
+		if (!tmp) return GLDI_NOTIFICATION_LET_PASS;
+		cd_debug ("got text");
+		// we will interpret the data as a list of filenames, so we always split by newline
+		data = g_strsplit ((gchar*)tmp, "\n", -1);
+		g_free (tmp);
+		// remove empty entries
+		unsigned int i, j = 0;
+		for (i = 0; data[i]; i++)
+		{
+			gboolean bIsSpace = TRUE;
+			gchar *tmp2 = data[i];
+			for(; *tmp2; ++tmp2)
+				if (! (*tmp2 == ' ' || *tmp2 == '\t' || *tmp2 == '\r'))
+					bIsSpace = FALSE;
+			if (bIsSpace) g_free (data[i]);
+			else
+			{
+				size_t len = tmp2 - data[i];
+				// we might get a set of strings delimited by "\r\n"
+				if (data[i][len - 1] == '\r') data[i][len - 1] = 0; // note: len > 0 if bIsSpace == FALSE
+				if (i != j) data[j] = data[i];
+				j++;
+			}
+		}
+		for (; data[j]; j++) data[j] = NULL;
+	}
+	
+	gboolean ret = GLDI_NOTIFICATION_LET_PASS;
+	gboolean bAllDesktop = TRUE;
+	{
+		gchar **tmp;
+		for (tmp = data; *tmp; ++tmp)
+			if (! g_str_has_suffix (*tmp, ".desktop"))
+			{
+				bAllDesktop = FALSE;
+				break;
+			}
+	}
+	if (bAllDesktop)
+	{
+		// all are .desktop files, add as launchers
+		CairoDock *pReceivingDock = pDock;
+		if (fOrder == CAIRO_DOCK_LAST_ORDER && icon && CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon)
+				&& icon->pSubDock != NULL)  // drop onto a container icon.
 		{
 			pReceivingDock = icon->pSubDock;  // -> add into the pointed sub-dock.
 		}
-	}
-	else  // file.
-	{
-		if (icon != NULL && fOrder == CAIRO_DOCK_LAST_ORDER)  // dropped on an icon
+		gchar **tmp;
+		for (tmp = data; *tmp; ++tmp)
 		{
-			if (CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon))  // sub-dock -> propagate to the sub-dock.
-			{
-				pReceivingDock = icon->pSubDock;
-			}
-			else if (CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (icon)
-			|| CAIRO_DOCK_ICON_TYPE_IS_APPLI (icon)
-			|| CAIRO_DOCK_ICON_TYPE_IS_CLASS_CONTAINER (icon)) // launcher/appli -> fire the command with this file.
-			{
-				if (icon->cCommand == NULL)
-					return GLDI_NOTIFICATION_LET_PASS;
-				gchar *cCommand;
-				if (strncmp (cReceivedData, "file://", 7) == 0)  // tous les programmes ne gerent pas les URI; pour parer au cas ou il ne le gererait pas, dans le cas d'un fichier local, on convertit en un chemin
-				{
-					gchar *cPath = g_filename_from_uri (cReceivedData, NULL, NULL);
-					cCommand = g_strdup_printf ("%s \"%s\"", icon->cCommand, cPath);
-					g_free (cPath);
-				}
-				else 
-					cCommand = g_strdup_printf ("%s \"%s\"", icon->cCommand, cReceivedData);
-				cd_message ("will open the file with the command '%s'...", cCommand);
-				g_spawn_command_line_async (cCommand, NULL);
-				g_free (cCommand);
-				gldi_icon_request_animation (icon, "blink", 2);
-				return GLDI_NOTIFICATION_INTERCEPT;
-			}
-			else  // skip any other case.
-			{
-				return GLDI_NOTIFICATION_LET_PASS;
-			}
-		}  // else: dropped between 2 icons -> try to add it (for instance a script).
+			// only add launchers if we got a valid .desktop file
+			gldi_launcher_add_new_full (*tmp, pReceivingDock, fOrder, TRUE);
+		}
+		ret = GLDI_NOTIFICATION_INTERCEPT;
+		*bHandled = TRUE;
 	}
-
-	if (g_bLocked || myDocksParam.bLockAll)
-		return GLDI_NOTIFICATION_LET_PASS;
-	
-	Icon *pNewIcon = gldi_launcher_add_new (cReceivedData, pReceivingDock, fOrder);
-	
-	return (pNewIcon ? GLDI_NOTIFICATION_INTERCEPT : GLDI_NOTIFICATION_LET_PASS);
+	else if (icon && icon->pAppInfo && (CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (icon)
+			|| CAIRO_DOCK_ICON_TYPE_IS_APPLI (icon)
+			|| CAIRO_DOCK_ICON_TYPE_IS_CLASS_CONTAINER (icon)))
+	{
+		gldi_app_info_launch (icon->pAppInfo, (const char * const*)data);
+		ret = GLDI_NOTIFICATION_INTERCEPT;
+		*bHandled = TRUE;
+	}
+	g_strfreev (data);
+	return ret;
 }
 
 
