@@ -150,8 +150,14 @@ struct _CairoDock {
 	gdouble fAlign;  // alignment, between 0 and 1, on the screen's edge.
 	/// visibility.
 	CairoDockVisibility iVisibility;
-	/// number of the screen the dock is placed on (-1 <=> all screen, >0 <=> num screen).
+	/// number of the screen the dock is actually placed on (between 0 and g_desktopGeometry.iNbScreens - 1)
+	/// might differ from iScreenReq if the requested screen is not available; filled out when updating the dock's position
 	gint iNumScreen;
+	/// number of the screen requested by the user
+	gint iScreenReq;
+	/// name of the screen requested by the user -- TODO: not used yet, should implement saving screen names !
+	gchar *cScreenReq;
+	
 	// icons
 	/// icon size, as specified in the config of the dock
 	gint iIconSize;
@@ -289,8 +295,14 @@ struct _CairoDock {
 	//\_______________ OpenGL.
 	GLuint iRedirectedTexture;
 	GLuint iFboId;
+	gboolean bNeedSizeUpdate; // set to TRUE whether we could not do size-related updated in the configure handler
+	/// last buffer scale factor set by us
+	gint iScaleFactor;
 	
-	gpointer reserved[4];
+	/// any data necessary for the dock visibility backend to work -- managed by the backend
+	gpointer pVisibilityData;
+	
+	gpointer reserved[3];
 };
 
 
@@ -328,6 +340,9 @@ CairoDock *gldi_dock_new (const gchar *cDockName);
 */
 CairoDock *gldi_subdock_new (const gchar *cDockName, const gchar *cRendererName, CairoDock *pParentDock, GList *pIconList);
 
+/* Deplace et redimensionne un dock a ses position et taille attitrees. Ne change pas la zone d'input (cela doit etre fait par ailleurs), et ne la replace pas (cela est fait lors du configure).
+*/
+void cairo_dock_move_resize_dock (CairoDock *pDock);
 
 /** Remove all icons from a dock (and its sub-docks). If the receiving dock is NULL, the icons are destroyed and removed from the current theme itself.
 *@param pDock a dock.
@@ -341,5 +356,21 @@ void cairo_dock_set_icon_size_in_dock (CairoDock *pDock, Icon *icon);
 
 void cairo_dock_create_redirect_texture_for_dock (CairoDock *pDock);
 
+/** Notify pDock that the mouse has possibly left it, just as if it
+*  received the "leave-notify-event" signal from GTK. Use this e.g.
+*  when a subdock, dialog or menu is closed and the dock should shrink
+*  down if the mouse is not over it.
+*@param pDock a dock.
+*/
+void gldi_dock_leave_synthetic (CairoDock *pDock);
+
+/** Notify pDock that the mouse has possibly entered it, just as if it
+*  received the "enter-notify-event" signal from GTK. Use this e.g.
+*  when a dialog is shown to prevent the dock from hiding.
+*@param pDock a dock.
+*/
+void gldi_dock_enter_synthetic (CairoDock *pDock);
+
+
 G_END_DECLS
 #endif
