@@ -17,6 +17,7 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#define _POSIX_C_SOURCE 200809L // needed for O_CLOEXEC
 #include <stdlib.h>      // atoi
 #include <string.h>      // memset
 #include <sys/stat.h>    // stat
@@ -35,8 +36,10 @@
 #include "cairo-dock-log.h"
 #include "cairo-dock-dock-manager.h"
 #include "cairo-dock-container.h"
-#include "cairo-dock-utils.h"  // cairo_dock_launch_command_sync, cairo_dock_property_is_present_on_root
+#include "cairo-dock-utils.h"  // cairo_dock_property_is_present_on_root
+#include "cairo-dock-class-manager.h"  // GldiAppInfo functions
 #include "cairo-dock-icon-manager.h"  // cairo_dock_free_icon
+#include "cairo-dock-menu.h" // gldi_menu_add_item
 #define _MANAGER_DEF_
 #include "cairo-dock-file-manager.h"
 
@@ -113,43 +116,18 @@ gboolean cairo_dock_fm_get_file_properties (const gchar *cURI, guint64 *iSize, t
 		return FALSE;
 }
 
-static gpointer _cairo_dock_fm_launch_uri_threaded (gchar *cURI)
-{
-	cd_debug ("%s (%s)", __func__, cURI);
-	s_pEnvBackend->launch_uri (cURI);
-	g_free (cURI);
-	return NULL;
-}
 gboolean cairo_dock_fm_launch_uri (const gchar *cURI)
 {
 	if (s_pEnvBackend != NULL && s_pEnvBackend->launch_uri != NULL && cURI != NULL)
 	{
-		// launch the URI in a thread.
-		//s_pEnvBackend->launch_uri (cURI);
-		GError *erreur = NULL;
-		gchar *cThreadURI = g_strdup (cURI);
-		#if (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 32)
-		GThread* pThread = g_thread_create ((GThreadFunc) _cairo_dock_fm_launch_uri_threaded, (gpointer) cThreadURI, FALSE, &erreur);
-		#else
-		// The name can be useful for discriminating threads in a debugger.
-		// Some systems restrict the length of name to 16 bytes. 
-		gchar *cThreadName = g_strndup (cURI, 15);
-		GThread* pThread = g_thread_try_new (cThreadName, (GThreadFunc) _cairo_dock_fm_launch_uri_threaded, (gpointer) cThreadURI, &erreur);
-		g_thread_unref (pThread);
-		g_free (cThreadName);
-		#endif
-		if (erreur != NULL)
-		{
-			cd_warning (erreur->message);
-			g_error_free (erreur);
-		}
+		s_pEnvBackend->launch_uri (cURI);
 		
 		// add it to the recent files.
 		GtkRecentManager *rm = gtk_recent_manager_get_default () ;
 		if (*cURI == '/')  // not an URI, this is now needed for gtk-recent.
 		{
 			gchar *cValidURI = g_filename_to_uri (cURI, NULL, NULL);
-			gtk_recent_manager_add_item (rm, cValidURI);
+			if (cValidURI) gtk_recent_manager_add_item (rm, cValidURI);
 			g_free (cValidURI);
 		}
 		else
@@ -490,8 +468,19 @@ gboolean cairo_dock_copy_file (const gchar *cFilePath, const gchar *cDestPath)
 {
 	gboolean ret = TRUE;
 	// open both files
-	int src_fd = open (cFilePath, O_RDONLY);
-	int dest_fd = open (cDestPath, O_CREAT | O_WRONLY, S_IRUSR|S_IWUSR | S_IRGRP | S_IROTH);  // mode=644
+	int src_fd = open (cFilePath, O_RDONLY | O_CLOEXEC);
+	int dest_fd;
+	if (g_file_test (cDestPath, G_FILE_TEST_IS_DIR))
+	{
+		const gchar *cFileName = strrchr(cFilePath, '/');
+		gchar *cFileDest = g_strdup_printf("%s/%s", cDestPath, cFileName ? cFileName : cFilePath);
+		dest_fd = open (cFileDest, O_CREAT | O_WRONLY | O_CLOEXEC, S_IRUSR|S_IWUSR | S_IRGRP | S_IROTH);  // mode=644
+		g_free(cFileDest);
+	}
+	else
+	{
+		dest_fd = open (cDestPath, O_CREAT | O_WRONLY | O_CLOEXEC, S_IRUSR|S_IWUSR | S_IRGRP | S_IROTH);  // mode=644
+	}
 	struct stat stat;
 	// get data size to be copied
 	if (fstat (src_fd, &stat) < 0)
@@ -538,6 +527,91 @@ gboolean cairo_dock_copy_file (const gchar *cFilePath, const gchar *cDestPath)
 }
 
 
+struct _submenu_data {
+	GList *data_list; // struct _launch_with_data
+	gchar *cPath;
+	gpointer user_data;
+	CairoDockFMOpenedWithCallback pCallback;
+};
+struct _launch_with_data {
+	GldiAppInfo *app;
+	struct _submenu_data *data;
+};
+
+static void _free_submenu_data_item (gpointer ptr)
+{
+	struct _launch_with_data *data = (struct _launch_with_data*)ptr;
+	gldi_object_unref (GLDI_OBJECT (data->app));
+	g_free (data);
+}
+static void _free_submenu_data (gpointer ptr, GObject*)
+{
+	struct _submenu_data *data = (struct _submenu_data*)ptr;
+	g_list_free_full (data->data_list, _free_submenu_data_item);
+	g_free (data->cPath);
+	g_free (data);
+}
+
+static void _submenu_launch_with (GtkMenuItem*, gpointer ptr)
+{
+	struct _launch_with_data *data = (struct _launch_with_data*)ptr;
+	const gchar *cURIs[] = {data->data->cPath, NULL};
+	gldi_app_info_launch (data->app, cURIs);
+	
+	if (data->data->pCallback) data->data->pCallback (data->data->user_data);
+}
+
+gboolean cairo_dock_fm_add_open_with_submenu (GList *pAppList, const gchar *cPath, GtkWidget *pMenu, const gchar *cLabel,
+	const gchar *cImage, CairoDockFMOpenedWithCallback pCallback, gpointer user_data)
+{
+	if (! (pAppList && cPath) ) return FALSE;
+	struct _submenu_data *data = g_new0 (struct _submenu_data, 1);
+	data->cPath = g_strdup (cPath);
+	data->user_data = user_data;
+	data->pCallback = pCallback;
+	
+	GtkWidget *pSubMenu = gldi_menu_add_sub_menu (pMenu, cLabel, cImage);
+	g_object_weak_ref (G_OBJECT (pSubMenu), _free_submenu_data, data);
+	
+	GList *a;
+	for (a = pAppList; a != NULL; a = a->next)
+	{
+		GAppInfo *app_info = a->data;
+		GDesktopAppInfo *desktop_app = G_DESKTOP_APP_INFO (app_info);
+		if (!desktop_app)
+		{
+			// should not happen, GDesktopAppInfo is the only implementation
+			cd_warning ("Unknown GAppInfo type!");
+			continue;
+		}
+		GldiAppInfo *gldi_app = gldi_app_info_from_desktop_app_info (desktop_app);
+		if (!gldi_app)
+		{
+			// warning already shown in gldi_app_info_from_desktop_app_info ()
+			continue;
+		}
+
+		gchar *cIconPath = NULL;
+		GIcon *pIcon = g_app_info_get_icon (app_info);
+		if (pIcon)
+		{
+			gchar *tmp = g_icon_to_string (pIcon);
+			cIconPath = cairo_dock_search_icon_s_path (tmp, cairo_dock_search_icon_size (GTK_ICON_SIZE_MENU));
+			g_free (tmp);
+		}
+
+		struct _launch_with_data *app_data = g_new (struct _launch_with_data, 1);
+		app_data->data = data;
+		app_data->app = gldi_app;
+		data->data_list = g_list_prepend (data->data_list, app_data); // save a reference to the app info
+
+		gldi_menu_add_item (pSubMenu, g_app_info_get_name (app_info), cIconPath, G_CALLBACK(_submenu_launch_with), app_data);
+		g_free (cIconPath);
+	}
+	return TRUE;
+}
+
+
   ///////////
  /// PID ///
 ///////////
@@ -545,14 +619,13 @@ gboolean cairo_dock_copy_file (const gchar *cFilePath, const gchar *cDestPath)
 int cairo_dock_fm_get_pid (const gchar *cProcessName)
 {
 	int iPID = -1;
-	gchar *cCommand = g_strdup_printf ("pidof %s", cProcessName);
-	gchar *cPID = cairo_dock_launch_command_sync (cCommand);
+	const char * const args[] = {"pidof", cProcessName, NULL};
+	gchar *cPID = cairo_dock_launch_command_argv_sync_with_stderr (args, TRUE);
 
 	if (cPID != NULL && *cPID != '\0')
 		iPID = atoi (cPID);
 
 	g_free (cPID);
-	g_free (cCommand);
 
 	return iPID;
 }
@@ -616,9 +689,26 @@ gboolean cairo_dock_fm_monitor_pid (const gchar *cProcessName, gboolean bCheckSa
  /// INIT ///
 ////////////
 
-static inline CairoDockDesktopEnv _guess_environment (void)
+static CairoDockDesktopEnv _guess_environment (void)
 {
-	const gchar * cEnv = g_getenv ("GNOME_DESKTOP_SESSION_ID");  // this value is now deprecated, but has been maintained for compatibility, so let's keep using it (Note: a possible alternative would be to check for org.gnome.SessionManager on Dbus)
+	// first, let's try out with XDG_CURRENT_DESKTOP
+	const gchar *cEnv = g_getenv ("XDG_CURRENT_DESKTOP");
+	if (cEnv != NULL)
+	{
+		if (strstr(cEnv, "GNOME") != NULL)
+			return CAIRO_DOCK_GNOME;
+		else if (strstr(cEnv, "XFCE") != NULL)
+			return CAIRO_DOCK_XFCE;
+		else if (strstr(cEnv, "KDE") != NULL)
+			return CAIRO_DOCK_KDE;
+		else
+		{
+			cd_debug ("couldn't interpret XDG_CURRENT_DESKTOP=%s", cEnv);
+		}
+	}
+	
+	// else, fall back to some old (less reliable) methods
+	cEnv = g_getenv ("GNOME_DESKTOP_SESSION_ID");  // this value is now deprecated, but has been maintained for compatibility, so let's keep using it (Note: a possible alternative would be to check for org.gnome.SessionManager on Dbus)
 	if (cEnv != NULL && *cEnv != '\0')
 		return CAIRO_DOCK_GNOME;
 	
@@ -635,7 +725,8 @@ static inline CairoDockDesktopEnv _guess_environment (void)
 		return CAIRO_DOCK_XFCE;
 	#endif
 	
-	gchar *cKWin = cairo_dock_launch_command_sync ("pgrep kwin");
+	const char * const args[] = {"pgrep", "kwin", NULL}; // will also match kwin_wayland, kwin_x11, etc.
+	gchar *cKWin = cairo_dock_launch_command_argv_sync_with_stderr (args, FALSE);
 	if (cKWin != NULL && *cKWin != '\0')
 	{
 		g_free (cKWin);
@@ -646,9 +737,20 @@ static inline CairoDockDesktopEnv _guess_environment (void)
 	return CAIRO_DOCK_UNKNOWN_ENV;
 	
 }
+
+const gchar *cairo_dock_fm_get_desktop_name (void)
+{
+	return (g_iDesktopEnv == CAIRO_DOCK_GNOME ? "GNOME" :
+		g_iDesktopEnv == CAIRO_DOCK_XFCE ? "XFCE" :
+		g_iDesktopEnv == CAIRO_DOCK_KDE ? "KDE" :
+		"unknown");
+}
+
 static void init (void)
 {
 	g_iDesktopEnv = _guess_environment ();
+	cd_message ("We found this desktop environment: %s",
+		cairo_dock_fm_get_desktop_name ());
 }
 
 
