@@ -17,7 +17,6 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-
 #ifndef __CAIRO_DOCK_FILE_MANAGER__
 #define  __CAIRO_DOCK_FILE_MANAGER__
 
@@ -32,22 +31,10 @@ G_BEGIN_DECLS
 * - the various desktop-related tools.
 */
 
-typedef struct _CairoDesktopEnvManager CairoDesktopEnvManager;
-
-#ifndef _MANAGER_DEF_
-extern CairoDesktopEnvManager myDesktopEnvMgr;
-#endif
-
-
 // manager
-struct _CairoDesktopEnvManager {
-	GldiManager mgr;
-	};
-
-// signals
-typedef enum {
-	NB_NOTIFICATIONS_DESKTOP_ENV = NB_NOTIFICATIONS_OBJECT
-	} CairoDesktopEnvNotifications;
+#ifndef _MANAGER_DEF_
+extern GldiManager myDesktopEnvMgr;
+#endif
 
 
 /// Type of available Desktop Environments.
@@ -215,7 +202,7 @@ gboolean cairo_dock_fm_move_file (const gchar *cURI, const gchar *cDirectoryURI)
 */
 gboolean cairo_dock_fm_create_file (const gchar *cURI, gboolean bDirectory);
 
-/** Get the list of applications that can open a given file. Returns a list of strings arrays : {name, command, icon}.
+/** Get the list of applications that can open a given file. Returns a list of GAppInfo
 */
 GList *cairo_dock_fm_list_apps_for_file (const gchar *cURI);
 
@@ -256,10 +243,10 @@ gboolean cairo_dock_fm_show_system_monitor (void);
 
 /** Create an Icon representing a given URI.
 */
-Icon *cairo_dock_fm_create_icon_from_URI (const gchar *cURI, CairoContainer *pContainer, CairoDockFMSortType iFileSortType);
+Icon *cairo_dock_fm_create_icon_from_URI (const gchar *cURI, GldiContainer *pContainer, CairoDockFMSortType iFileSortType);
 
 
-gboolean cairo_dock_fm_move_into_directory (const gchar *cURI, Icon *icon, CairoContainer *pContainer);
+gboolean cairo_dock_fm_move_into_directory (const gchar *cURI, Icon *icon, GldiContainer *pContainer);
 
 
 /** Get the size of a local file.
@@ -271,7 +258,48 @@ int cairo_dock_get_file_size (const gchar *cFilePath);
 gboolean cairo_dock_copy_file (const gchar *cFilePath, const gchar *cDestPath);
 
 
+/** Get process ID given its name
+ * @param cProcessName name of the process
+ * @return the PID if it exists or -1
+ */
+int cairo_dock_fm_get_pid (const gchar *cProcessName);
+
+/** Monitor a process. Call a function when the process is no longer running
+ * @param cProcessName name(es) of the process(es)
+ * @param bCheckSameProcess TRUE to check if first match is running. FALSE to
+ *        check every time if this process name is running even if it's not the
+ *        same PID.
+ * @param pCallback function to call when the process is no longer running
+ * @param bAlwaysLaunch TRUE to launch the callback function even if the process
+ *        is not running or if there is an error
+ * @param pUserData data to pass to pCallback
+ * @return FALSE if the process is not running or if there is an error
+ */
+gboolean cairo_dock_fm_monitor_pid (const gchar *cProcessName, gboolean bCheckSameProcess, GSourceFunc pCallback, gboolean bAlwaysLaunch, gpointer pUserData);
+
+const gchar *cairo_dock_fm_get_desktop_name (void);
+
 void gldi_register_desktop_environment_manager (void);
 
+typedef void (*CairoDockFMOpenedWithCallback) (gpointer ptr);
+/** Create a submenu for presenting options to open a file and add it to the given menu.
+ * @param pAppList a list with GAppInfo elements to add to the submenu (e.g. the return
+ *        value of cairo_dock_fm_list_apps_for_file ()). Owned by the caller, but apps
+ *        added to the menu will be refed, so can be freed.
+ * @param cPath path of the file to open
+ * @param pMenu menu to add a submenu to
+ * @param cLabel label of the submenu item
+ * @param cImage stock image to use with the label
+ * @param pCallback an optional callback function to call when the app was launched (can be NULL)
+ * @param user_data data to pass to pCallback
+ * @return TRUE if the submenu was successfully created and added to pMenu
+ * 
+ * Note: the created submenu is managed internally and will be freed when the menu is destroyed. If given,
+ * user_data should remain valid while the menu is open. The callback function is only called if the app
+ * is launched, so it is possible that it is never called.
+ */
+gboolean cairo_dock_fm_add_open_with_submenu (GList *pAppList, const gchar *cPath, GtkWidget *pMenu, const gchar *cLabel,
+	const gchar *cImage, CairoDockFMOpenedWithCallback pCallback, gpointer user_data);
+
 G_END_DECLS
 #endif
