@@ -24,7 +24,7 @@
 
 #include "cairo-dock-struct.h"
 #include "cairo-dock-manager.h"
-#include "cairo-dock-surface-factory.h"  // CairoDockLabelDescription
+#include "cairo-dock-style-facility.h"  // GldiTextDescription
 #include "cairo-dock-icon-factory.h"  // CairoDockIconGroup
 G_BEGIN_DECLS
 
@@ -34,12 +34,13 @@ G_BEGIN_DECLS
 * Specialized Icons are handled by the corresponding manager.
 */
 
+// manager
 typedef struct _CairoIconsParam CairoIconsParam;
-typedef struct _CairoIconsManager CairoIconsManager;
 
 #ifndef _MANAGER_DEF_
 extern CairoIconsParam myIconsParam;
-extern CairoIconsManager myIconsMgr;
+extern GldiManager myIconsMgr;
+extern GldiObjectManager myIconObjectMgr;
 #endif
 
 #define CAIRO_DOCK_DEFAULT_ICON_SIZE 128
@@ -50,7 +51,7 @@ typedef enum {
 	CAIRO_DOCK_FLAT_SEPARATOR,
 	CAIRO_DOCK_PHYSICAL_SEPARATOR,
 	CAIRO_DOCK_NB_SEPARATOR_TYPES
-	} CairoDockSpeparatorType;
+	} CairoDockSeparatorType;
 
 struct _CairoIconsParam {
 	// icons
@@ -69,31 +70,22 @@ struct _CairoIconsParam {
 	gint iIconWidth;  // default icon size
 	gint iIconHeight;
 	// separators
-	CairoDockSpeparatorType iSeparatorType;
+	CairoDockSeparatorType iSeparatorType;
 	gint iSeparatorWidth;
 	gint iSeparatorHeight;
 	gchar *cSeparatorImage;
 	gboolean bRevolveSeparator;
 	gboolean bConstantSeparatorSize;
-	gdouble fSeparatorColor[4];
+	gboolean bSeparatorUseDefaultColors;
+	GldiColor fSeparatorColor;
 	// labels
-	CairoDockLabelDescription iconTextDescription;
-	CairoDockLabelDescription quickInfoTextDescription;
+	GldiTextDescription iconTextDescription;
+	GldiTextDescription quickInfoTextDescription;
 	gboolean bLabelForPointedIconOnly;
 	gint iLabelSize;  // taille des etiquettes des icones, en prenant en compte le contour et la marge.
 	gdouble fLabelAlphaThreshold;
-	gboolean bTextAlwaysHorizontal;
 	};
 
-// manager
-struct _CairoIconsManager {
-	GldiManager mgr;
-	void (*free_icon) (Icon *icon);
-	void (*foreach_icons) (CairoDockForeachIconFunc pFunction, gpointer pUserData);
-	void (*hide_show_launchers_on_other_desktops) (CairoDock *pDock);
-	void (*set_specified_desktop_for_icon) (Icon *pIcon, int iSpecificDesktop);
-	} ;
-
 /// signals
 typedef enum {
 	/// notification called when an icon's sub-dock is starting to (un)fold. data : {Icon}
@@ -115,19 +107,12 @@ typedef enum {
 	} CairoIconNotifications;
 
 
-/** Terminate an Icon and free all its allocated ressources, except its sub-dock.
-*@param icon the icon to destroy.
-*/
-void cairo_dock_free_icon (Icon *icon);
-
-void cairo_dock_delete_icon_from_current_theme (Icon *icon);
-
 
 /** Execute an action on all icons.
 *@param pFunction the action.
 *@param pUserData data passed to the callback.
 */
-void cairo_dock_foreach_icons (CairoDockForeachIconFunc pFunction, gpointer pUserData);
+void gldi_icons_foreach (GldiIconFunc pFunction, gpointer pUserData);
 
 
 void cairo_dock_hide_show_launchers_on_other_desktops (void);
