@@ -22,9 +22,7 @@
 
 #include <cairo.h>
 #include <gtk/gtk.h>
-#if GTK_CHECK_VERSION (3, 10, 0)
 #include "gtk3imagemenuitem.h"
-#endif
 
 #include "cairo-dock-container.h"
 #include "cairo-dock-icon-factory.h"
@@ -36,15 +34,12 @@
 #include "cairo-dock-dialog-manager.h"  // myDialogsParam
 #include "cairo-dock-style-manager.h"
 #include "cairo-dock-menu.h"
+#include "cairo-dock-wayland-manager.h"
 
 extern gchar *g_cCurrentThemePath;
+extern GldiContainer *g_pPrimaryContainer;
 
-/*
 static gboolean _draw_menu_item (GtkWidget *widget, cairo_t *cr, G_GNUC_UNUSED gpointer data);
-static gboolean _on_select_menu_item (GtkWidget* pMenuItem, G_GNUC_UNUSED gpointer data);
-static gboolean _on_deselect_menu_item (GtkWidget* pMenuItem, G_GNUC_UNUSED gpointer data);
-static void _on_destroy_menu_item (GtkWidget* pMenuItem, G_GNUC_UNUSED gpointer data);
-*/
 
   ////////////
  /// MENU ///
@@ -84,13 +79,20 @@ void _init_menu_style (void)
 		}
 		
 		// css header: define colors from the global style
-		GldiColor *bg_color = (myDialogsParam.bUseDefaultColors ? &myStyleParam.fBgColor : &myDialogsParam.fBgColor);
-		GldiColor *text_color = (myDialogsParam.bUseDefaultColors ? &myStyleParam.textDescription.fColorStart : &myDialogsParam.dialogTextDescription.fColorStart);
-		
+		GldiColor bg_color;
+		if (myDialogsParam.bUseDefaultColors)
+			gldi_style_color_get (GLDI_COLOR_BG, &bg_color);
+		else
+			bg_color = myDialogsParam.fBgColor;
+		GldiColor text_color;
+		if (myDialogsParam.bUseDefaultColors)
+			gldi_style_color_get (GLDI_COLOR_TEXT, &text_color);
+		else
+			text_color = myDialogsParam.dialogTextDescription.fColorStart;
 		GldiColor rgb;  // menuitem bg color: a little darker/lighter than the menu's bg color; also separator color (with no alpha)
-		gldi_style_color_shade (bg_color, GLDI_COLOR_SHADE_MEDIUM, &rgb);
+		gldi_style_color_shade (&bg_color, GLDI_COLOR_SHADE_MEDIUM, &rgb);
 		GldiColor rgbb;  // menuitem border color and menuitem's child bg color (for instance, calendar, scale, etc): a little darker/lighter than the menuitem bg color
-		gldi_style_color_shade (bg_color, GLDI_COLOR_SHADE_STRONG, &rgbb);
+		gldi_style_color_shade (&bg_color, GLDI_COLOR_SHADE_STRONG, &rgbb);
 		
 		gchar *cssheader = g_strdup_printf ("@define-color menuitem_bg_color rgba (%d, %d, %d, %f); \n\
 		@define-color menuitem_text_color rgb (%d, %d, %d); \n\
@@ -99,11 +101,11 @@ void _init_menu_style (void)
 		@define-color menuitem_child_bg_color rgba (%d, %d, %d, %f); \n\
 		@define-color menu_bg_color rgba (%d, %d, %d, %f);\n",
 			(int)(rgb.rgba.red*255), (int)(rgb.rgba.green*255), (int)(rgb.rgba.blue*255), rgb.rgba.alpha,
-			(int)(text_color->rgba.red*255), (int)(text_color->rgba.green*255), (int)(text_color->rgba.blue*255),
-			(int)(text_color->rgba.red*255), (int)(text_color->rgba.green*255), (int)(text_color->rgba.blue*255),
+			(int)(text_color.rgba.red*255), (int)(text_color.rgba.green*255), (int)(text_color.rgba.blue*255),
+			(int)(text_color.rgba.red*255), (int)(text_color.rgba.green*255), (int)(text_color.rgba.blue*255),
 			(int)(rgb.rgba.red*255), (int)(rgb.rgba.green*255), (int)(rgb.rgba.blue*255),
 			(int)(rgbb.rgba.red*255), (int)(rgbb.rgba.green*255), (int)(rgbb.rgba.blue*255), rgbb.rgba.alpha,
-			(int)(bg_color->rgba.red*255), (int)(bg_color->rgba.green*255), (int)(bg_color->rgba.blue*255), bg_color->rgba.alpha);
+			(int)(bg_color.rgba.red*255), (int)(bg_color.rgba.green*255), (int)(bg_color.rgba.blue*255), bg_color.rgba.alpha);
 		
 		// css body: load a custom file if it exists
 		gchar *cCustomCss = NULL;
@@ -144,7 +146,8 @@ void _init_menu_style (void)
 				-unico-glow-color: transparent; \
 				-unico-glow-radius: 0; \
 			} \
-			.gldimenuitem GtkImage { \
+			.gldimenuitem GtkImage, \
+			.gldimenuitem .image { \
 				background: transparent; \
 			} \
 			.gldimenuitem.separator, \
@@ -168,11 +171,10 @@ void _init_menu_style (void)
 				color: @menuitem_text_color; \
 				border-radius: 5px; \
 				border-style: solid; \
-				border-width: 1px; \
 				border-color: @menuitem_child_bg_color; \
 				-unico-inner-stroke-color: transparent; \
 			} \
-			.gldimenuitem *:insensitive { \
+			.gldimenuitem *:disabled { \
 				text-shadow: none; \
 				color: @menuitem_insensitive_text_color; \
 				background: transparent; \
@@ -245,7 +247,7 @@ void _init_menu_style (void)
 				background-color: @menuitem_child_bg_color; \
 				background-image: none; \
 			} \
-			.gldimenuitem GtkCalendar:inconsistent { \
+			.gldimenuitem GtkCalendar:indeterminate { \
 				color: shade (@menuitem_child_bg_color, 0.6); \
 			} \
 			.gldimenuitem .toolbar .button, \
@@ -290,9 +292,13 @@ void _init_menu_style (void)
 				background-color: @menu_bg_color; \
 				background-image: none; \
 				color: @menuitem_text_color; \
+			} \
+			.window-frame { \
+				box-shadow: none; \
 			}",
 			NULL);  // we also define ".menu", so that custom widgets (like in the SoundMenu) can get our colors. Note that we don't redefine Gtk's menuitem, because we want to keep normal menus for GUI
 			// for "entry", using "background-color" will not affect entries inside another widget (like a box), we actually have to use "background" ... (TBC with gtk > 3.6)
+			// for ".window-frame": remove shadow added by some WMs (Marco/Metacity) to the menu (LP #1407880)
 		}
 		
 		gldi_style_colors_freeze ();
@@ -307,6 +313,9 @@ static gboolean _draw_menu (GtkWidget *pWidget,
 	cairo_t *pCairoContext,
 	G_GNUC_UNUSED GtkWidget *menu)
 {
+	// reset the clip set by GTK, to allow us draw in the margin of the widget
+	cairo_reset_clip(pCairoContext);
+	
 	// erase the default background
 	cairo_dock_erase_cairo_context (pCairoContext);
 	
@@ -337,17 +346,35 @@ static void _set_margin_position (GtkWidget *pMenu, GldiMenuParams *pParams)
 	
 	// define where the menu will point
 	int iMarginPosition;  // b, t, r, l
-	int y0 = pContainer->iWindowPositionY + pIcon->fDrawY;
-	if (pContainer->bDirectionUp)
-		y0 += pIcon->fHeight * pIcon->fScale - pIcon->image.iHeight;  // the icon might not be maximised yet
-	int Hs = (pContainer->bIsHorizontal ? gldi_desktop_get_height() : gldi_desktop_get_width());
-	if (pContainer->bIsHorizontal)
+	// new positioning code should work on both X11 and Wayland, but, by default, it is used
+	// only on Wayland, unless it is specifically requested by the user
+	if (gldi_container_use_new_positioning_code ())
 	{
-		iMarginPosition = (y0 > Hs/2 ? 0 : 1);
+		if (pContainer->bIsHorizontal)
+		{
+			if (pContainer->bDirectionUp) iMarginPosition = 0;
+			else iMarginPosition = 1;
+		}
+		else
+		{
+			if (pContainer->bDirectionUp) iMarginPosition = 2;
+			else iMarginPosition = 3;
+		}
 	}
 	else
 	{
-		iMarginPosition = (y0 > Hs/2 ? 2 : 3);
+		int y0 = pContainer->iWindowPositionY + pIcon->fDrawY;
+		if (pContainer->bDirectionUp)
+			y0 += pIcon->fHeight * pIcon->fScale - pIcon->image.iHeight;  // the icon might not be maximised yet
+		int Hs = (pContainer->bIsHorizontal ? gldi_desktop_get_height() : gldi_desktop_get_width());
+		if (pContainer->bIsHorizontal)
+		{
+			iMarginPosition = (y0 > Hs/2 ? 0 : 1);
+		}
+		else
+		{
+			iMarginPosition = (y0 > Hs/2 ? 2 : 3);
+		}
 	}
 	
 	// store the result, and allocate some space to draw the arrow
@@ -356,7 +383,10 @@ static void _set_margin_position (GtkWidget *pMenu, GldiMenuParams *pParams)
 		// store the value
 		pParams->iMarginPosition = iMarginPosition;
 		
-		// get/add a css (gtk_widget_set_margin_xxx doesn't work as expected :-/)
+		// get/add a css
+		// actually gtk_widget_set_margin_xxx works, but then GTK adds a translation to the cairo_context, forcing each renderer to offset its drawing by gtk_widget_get_margin_xxx()
+		// also, gtk_widget_get_allocation() doesn't take into account the margin, forcing each renderer to add it
+		// so in the end it's better not to use it
 		GtkCssProvider *cssProvider = pParams->cssProvider;
 		if (cssProvider)  // unfortunately, GTK doesn't update correctly a css provider if we load some new data inside (the previous padding values are kept, along with the new ones, although 'gtk_css_provider_load_from_data' is supposed to "clear any previously loaded information"), so we have to remove/add it :-/
 		{
@@ -384,12 +414,12 @@ static void _set_margin_position (GtkWidget *pMenu, GldiMenuParams *pParams)
 			case 3: l = ah; break;
 			default: break;
 		}
-		css = g_strdup_printf ("GtkMenu { \
+		css = g_strdup_printf ("GtkMenu,menu { \
 			padding-bottom: %dpx; \
 			padding-top: %dpx; \
 			padding-right: %dpx; \
 			padding-left: %dpx; \
-		}", b, t, r, l);  // we must define all the paddings, else if the margin position changes, clearing the css won't make the padding disappear :-/
+		}", b, t, r, l);  // we must define all the paddings, else if the margin position changes, clearing the css won't make the padding disappear; also, 'GtkMenu' is old
 		gtk_css_provider_load_from_data (cssProvider,
 			css, -1, NULL);
 		g_free (css);
@@ -410,6 +440,8 @@ static gboolean _on_icon_destroyed (GtkWidget *pMenu, G_GNUC_UNUSED Icon *pIcon)
 	GldiMenuParams *pParams = g_object_get_data (G_OBJECT (pMenu), "gldi-params");
 	if (pParams)
 		pParams->pIcon = NULL;
+	// no sense in keeping the menu (and it might hold more stale references to pIcon)
+	gtk_widget_destroy (pMenu);
 	return GLDI_NOTIFICATION_LET_PASS;
 }
 
@@ -442,13 +474,22 @@ static void _on_menu_deactivated (GtkMenuShell *pMenu, G_GNUC_UNUSED gpointer da
 	if (!pParams)
 		return;
 	Icon *pIcon = pParams->pIcon;
+	if (!pIcon) return;
+	GldiContainer *pContainer = cairo_dock_get_icon_container (pIcon);
 	if (pIcon->iHideLabel > 0)
 	{
 		pIcon->iHideLabel --;
-		GldiContainer *pContainer = cairo_dock_get_icon_container (pIcon);
 		if (pIcon->iHideLabel == 0 && pContainer)
 			gtk_widget_queue_draw (pContainer->pWidget);
 	}
+	// no need to test if we're running on Wayland, if not, this is a no-op
+	gldi_wayland_release_keyboard (pContainer, GLDI_KEYBOARD_RELEASE_MENU_CLOSED);
+}
+
+static void _menu_realized_cb (GtkWidget *widget, gpointer user_data);
+static void _menu_popped_up_cb (GtkWidget *widget, void*, void*, GdkGravity, GdkGravity, gpointer user_data)
+{
+	_menu_realized_cb (widget, user_data);
 }
 
 void gldi_menu_init (GtkWidget *pMenu, Icon *pIcon)
@@ -477,7 +518,18 @@ void gldi_menu_init (GtkWidget *pMenu, Icon *pIcon)
 		"destroy",
 		G_CALLBACK (_on_menu_destroyed),
 		NULL);
-		
+	
+	// handle any adjustments necessary when the menu is first shown
+	if (gldi_container_use_new_positioning_code ())
+	{
+		// This is a bit hacky: if we have an icon (so we are the first-level menu), we connect to the "realized" signal,
+		// which will allow us to resize the menu before it is positioned, leading to better results if it needs to be moved out
+		// of the way from the dock it's pointing at. For submenus, we connect to the "popped-up" signal, to better handle it
+		// opening and closing multiple times.
+		if (pIcon) g_signal_connect (G_OBJECT (pMenu), "realize", G_CALLBACK (_menu_realized_cb), pParams);
+		else g_signal_connect (G_OBJECT (pMenu), "popped-up", G_CALLBACK (_menu_popped_up_cb), pParams);
+	}
+	
 	// init a main menu
 	if (pIcon != NULL)  // the menu points on an icon
 	{
@@ -507,30 +559,67 @@ void gldi_menu_init (GtkWidget *pMenu, Icon *pIcon)
 				G_CALLBACK (_on_menu_deactivated),
 				NULL);
 			
-			gchar *css = NULL;
-			const gchar *orientation = "";
-			int ah = pParams->iArrowHeight;
-			switch (pParams->iMarginPosition)
+			// set transient for (parent relationship; needed for positioning on Wayland)
+			// note: it is an error to try to map (and position) a popup
+			// relative to a window that is not mapped; we need to take care of this
+			if (gldi_container_use_new_positioning_code ())
 			{
-				case 0: orientation = "bottom";  break;
-				case 1: orientation = "top";  break;
-				case 2: orientation = "right";  break;
-				case 3: orientation = "left";  break;
-				default: break;
+				GtkWindow *tmp = GTK_WINDOW (pContainer->pWidget);
+				while (tmp && !gtk_widget_get_mapped (GTK_WIDGET(tmp)))
+					tmp = gtk_window_get_transient_for (tmp);
+				gtk_window_set_transient_for (GTK_WINDOW (pWindow), tmp);
 			}
-			
-			GtkCssProvider *cssProvider = gtk_css_provider_new ();
-			css = g_strdup_printf ("GtkMenu { \
-				padding-%s: %dpx; \
-			}", orientation, ah);
-			gtk_css_provider_load_from_data (cssProvider,
-				css, -1, NULL);
-			gtk_style_context_add_provider (gtk_widget_get_style_context(pMenu), GTK_STYLE_PROVIDER(cssProvider), GTK_STYLE_PROVIDER_PRIORITY_USER);
-			g_free (css);
 		}
 	}
 }
 
+
+static void _menu_realized_cb (GtkWidget *widget, gpointer user_data)
+{
+	GldiMenuParams *pParams = (GldiMenuParams*)user_data;
+	g_return_if_fail (pParams != NULL);
+	
+	int w, h;  // taille menu
+	GtkRequisition requisition;
+	gtk_widget_get_preferred_size (widget, NULL, &requisition);  // retrieve the natural size; Note: before gtk3.10 we used the minimum size but it's now incorrect; the natural size works for prior versions too.
+	w = requisition.width;
+	h = requisition.height;
+	
+	// constrain the menu's size to fit within the screen
+	// see https://github.com/wmww/gtk-layer-shell/issues/148
+	// (note: menus created here are always ultimately a child of a layer-shell window)
+	// solution used here is originally by LBCrion (under GPL3):
+	// https://github.com/LBCrion/sfwbar/commit/760e68ef50c540a55c13791876f853d358b4dcaa
+	if (gldi_wayland_manager_have_layer_shell ())
+	{
+		GdkRectangle area;
+		GdkWindow *window = gtk_widget_get_window (gtk_widget_get_toplevel (widget));
+		if (window)
+		{
+			GdkDisplay *dsp = gdk_window_get_display (window);
+			GdkMonitor *mon = gdk_display_get_monitor_at_window (dsp, window);
+			gdk_monitor_get_workarea (mon, &area);
+			gboolean bResize = FALSE;
+			if (w > area.width)
+			{
+				w = area.width;
+				bResize = TRUE;
+			}
+			if (h > area.height)
+			{
+				h = area.height;
+				bResize = TRUE;
+			}
+			if (bResize) gdk_window_resize (window, w, h);
+		}
+		else cd_warning ("menu has no associated GdkWindow!");
+	}
+	
+	if (pParams->pIcon && gldi_container_use_new_positioning_code ())
+		gldi_container_calculate_aimed_point (pParams->pIcon, widget, w, h, pParams->iMarginPosition,
+			pParams->fAlign, &(pParams->iAimedX), &(pParams->iAimedY));
+}
+
 static void _place_menu_on_icon (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, G_GNUC_UNUSED gpointer data)
 {
 	*push_in = FALSE;
@@ -595,6 +684,13 @@ static void _place_menu_on_icon (GtkMenu *menu, gint *x, gint *y, gboolean *push
 	pParams->iAimedY = iAimedY;
 }
 
+
+static void _init_menu_item (GtkWidget *pMenuItem);
+static void _init_menu_item2 (GtkWidget *menu, gpointer)
+{
+	_init_menu_item (menu);
+}
+
 static void _init_menu_item (GtkWidget *pMenuItem)
 {
 	GtkWidget *pSubMenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (pMenuItem));
@@ -603,23 +699,11 @@ static void _init_menu_item (GtkWidget *pMenuItem)
 	gboolean bStyleIsSet = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pMenuItem), "gldi-style-set"));
 	if (! bStyleIsSet)  // not done yet -> do it once
 	{
-		// the following code is to draw the menu item; this is more like a proof of concept
-		/**g_signal_connect (G_OBJECT (pMenuItem),
+		// draw the menu items; actually, we only want to draw the separators, which are not well drawn by GTK
+		g_signal_connect (G_OBJECT (pMenuItem),
 			"draw",
 			G_CALLBACK (_draw_menu_item),
 			NULL);
-		g_signal_connect (G_OBJECT (pMenuItem),
-			"select",
-			G_CALLBACK (_on_select_menu_item),
-			NULL);
-		g_signal_connect (G_OBJECT (pMenuItem),
-			"deselect",
-			G_CALLBACK (_on_deselect_menu_item),
-			NULL);
-		g_signal_connect (G_OBJECT (pMenuItem),
-			"destroy",
-			G_CALLBACK (_on_destroy_menu_item),
-			NULL);*/
 		
 		gtk_style_context_add_class (gtk_widget_get_style_context (pMenuItem), "gldimenuitem");
 		
@@ -632,10 +716,31 @@ static void _init_menu_item (GtkWidget *pMenuItem)
 	
 	// iterate on sub-menu's items
 	if (pSubMenu != NULL)
-		gtk_container_forall (GTK_CONTAINER (pSubMenu), (GtkCallback) _init_menu_item, NULL);
+		gtk_container_forall (GTK_CONTAINER (pSubMenu), (GtkCallback) _init_menu_item2, NULL);
 }
 
-static void _popup_menu (GtkWidget *menu, guint32 time)
+static void _adjust_anchor (GdkGravity *anchor, gboolean bLeft)
+{
+	switch (*anchor)
+	{
+		case GDK_GRAVITY_SOUTH:
+			*anchor = bLeft ? GDK_GRAVITY_SOUTH_WEST : GDK_GRAVITY_SOUTH_EAST;
+			break;
+		case GDK_GRAVITY_NORTH:
+			*anchor = bLeft ? GDK_GRAVITY_NORTH_WEST : GDK_GRAVITY_NORTH_EAST;
+			break;
+		case GDK_GRAVITY_EAST:
+			*anchor = bLeft ? GDK_GRAVITY_NORTH_EAST : GDK_GRAVITY_SOUTH_EAST;
+			break;
+		case GDK_GRAVITY_WEST:
+			*anchor = bLeft ? GDK_GRAVITY_NORTH_WEST : GDK_GRAVITY_SOUTH_WEST;
+			break;
+		default:
+			break;
+	}
+}
+
+static void _popup_menu (GtkWidget *menu, const GdkEvent *event)
 {
 	GldiMenuParams *pParams = g_object_get_data (G_OBJECT(menu), "gldi-params");
 	g_return_if_fail (pParams != NULL);
@@ -648,7 +753,7 @@ static void _popup_menu (GtkWidget *menu, guint32 time)
 		pContainer->iface.setup_menu (pContainer, pIcon, menu);
 
 	// init each items (and sub-menus), in case it contains some foreign GtkMenuItems (for instance in case of an indicator menu or the gtk recent files sub-menu, which can have new items at any time)
-	gtk_container_forall (GTK_CONTAINER (menu), (GtkCallback) _init_menu_item, NULL);  // init each menu-item style
+	gtk_container_forall (GTK_CONTAINER (menu), (GtkCallback) _init_menu_item2, NULL);  // init each menu-item style
 	
 	if (pIcon && pContainer)
 	{
@@ -663,29 +768,88 @@ static void _popup_menu (GtkWidget *menu, guint32 time)
 
 	gtk_widget_show_all (GTK_WIDGET (menu));
 
-	gtk_menu_popup (GTK_MENU (menu),
-		NULL,
-		NULL,
-		pIcon != NULL && pContainer != NULL ? _place_menu_on_icon : NULL,
-		NULL,
-		0,
-		time);
+	// new positioning code should work on both X11 and Wayland, but, by default, it is used
+	// only on Wayland, unless it is specifically requested by the user
+	if (gldi_container_use_new_positioning_code ())
+	{
+		if (pContainer && pIcon)
+		{
+			GdkRectangle rect = {0, 0, 1, 1};
+			GdkGravity rect_anchor = GDK_GRAVITY_NORTH;
+			GdkGravity menu_anchor = GDK_GRAVITY_SOUTH;
+			gldi_container_calculate_rect (pContainer, pIcon, &rect, &rect_anchor, &menu_anchor, FALSE);
+			
+			if (pParams->fAlign == 0.0 || pParams->fAlign == 1.0)
+			{
+				// adjust anchors
+				_adjust_anchor (&rect_anchor, (pParams->fAlign == 0.0));
+				_adjust_anchor (&menu_anchor, (pParams->fAlign == 0.0));
+			}
+			else
+			{
+				/* add an offset -- unfortunately, we can only add an absolute offset, but we
+				 * do not know our size, and by the time we get it in _menu_realized_cb (),
+				 * setting an offset does not have an effect
+				 * see e.g. (links to the latest GTK+3 release when writing this):
+https://gitlab.gnome.org/GNOME/gtk/-/blob/e1d664da630ee32c4068c8ead4101bce94e7e24a/gtk/gtkmenu.c#L5218
+https://gitlab.gnome.org/GNOME/gtk/-/blob/e1d664da630ee32c4068c8ead4101bce94e7e24a/gtk/gtkmenu.c#L5303
+https://gitlab.gnome.org/GNOME/gtk/-/blob/e1d664da630ee32c4068c8ead4101bce94e7e24a/gtk/gtkmenu.c#L5325
+				 * so we just use a dummy size that works in most cases (note: this is only
+				 * used by the "modern" renderer, all others have fAlign == 0.0, 0.5 or 1.0
+				 * which is handled correctly by setting anchors)
+				 */
+				const double dummy_width = 240.0;
+				const double dummy_height = 120.0;
+				if (pContainer->bIsHorizontal)
+				{
+					int dx = (int) (dummy_width * (0.5 - pParams->fAlign));
+					g_object_set (G_OBJECT (menu), "rect-anchor-dx", dx, NULL);
+				}
+				else
+				{
+					int dy = (int) (dummy_height * (0.5 - pParams->fAlign));
+					g_object_set (G_OBJECT (menu), "rect-anchor-dy", dy, NULL);
+				}
+			}
+			
+			gtk_menu_popup_at_rect (GTK_MENU (menu), gtk_widget_get_window (pContainer->pWidget),
+				&rect, rect_anchor, menu_anchor, event);
+		}
+		else
+		{
+			gtk_menu_popup_at_pointer (GTK_MENU (menu), event);
+		}
+	}
+	else
+	{
+		guint button;
+		if ( !(event && gdk_event_get_button (event, &button))) button = 0;
+		gtk_menu_popup (GTK_MENU (menu),
+			NULL,
+			NULL,
+			pIcon != NULL && pContainer != NULL ? _place_menu_on_icon : NULL,
+			NULL,
+			button,
+			gdk_event_get_time (event)); // note: event can be NULL, in this case, the current time is used
+	}
 }
 static gboolean _popup_menu_delayed (GtkWidget *menu)
 {
-	_popup_menu (menu, 0);
+	_popup_menu (menu, NULL);
 	return FALSE;
 }
-void gldi_menu_popup (GtkWidget *menu)
+void gldi_menu_popup_full (GtkWidget *menu, const GdkEvent *event)
 {
 	if (menu == NULL)
 		return;
 	
-	guint32 t = gtk_get_current_event_time();
-	cd_debug ("gtk_get_current_event_time: %d", t);
-	if (t > 0)
+	GdkEvent *currentEvent = NULL;
+	if (!event) event = currentEvent = gtk_get_current_event ();
+	
+	if (event)
 	{
-		_popup_menu (menu, t);
+		_popup_menu (menu, event);
+		if (currentEvent) gdk_event_free (currentEvent);
 	}
 	else  // 'gtk_menu_popup' is buggy and doesn't work if not triggered directly by an X event :-/ so in this case, we run it with a delay (200ms is the minimal value that always works).
 	{
@@ -698,270 +862,91 @@ void gldi_menu_popup (GtkWidget *menu)
  /// MENU ITEM ///
 /////////////////
 
-const int N = 10;
-const int dt1 = 20;
-const int dt2 = 30;
-
-/*
-static void
-get_arrow_size (GtkWidget *widget,
-                GtkWidget *child,
-                gint      *size,
-                gint      *spacing)
-{
-	PangoContext     *context;
-	PangoFontMetrics *metrics;
-	gfloat            arrow_scaling;
-	gint              arrow_spacing;
-
-	g_assert (size);
-
-	gtk_widget_style_get (widget,
-		"arrow-scaling", &arrow_scaling,
-		"arrow-spacing", &arrow_spacing,
-		NULL);
-
-	if (spacing != NULL)
-		*spacing = arrow_spacing;
-
-	context = gtk_widget_get_pango_context (child);
-
-	metrics = pango_context_get_metrics (context,
-	                                     pango_context_get_font_description (context),
-	                                     pango_context_get_language (context));
-
-	*size = (PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) +
-	                       pango_font_metrics_get_descent (metrics)));
-
-	pango_font_metrics_unref (metrics);
-
-	*size = *size * arrow_scaling;
-}
 
 static gboolean _draw_menu_item (GtkWidget *widget,
 	cairo_t *cr,
 	G_GNUC_UNUSED gpointer data)
 {
-	//GtkStateFlags state;
-	GtkStyleContext *context;
-	GtkWidget *child;
-	gint x, y, w, h, width, height;
+	if (! GTK_IS_SEPARATOR_MENU_ITEM(widget))  // not a separator => skip drawing anything and let GTK handle it
+		return FALSE;
+	
+	// get menu's geometry
 	guint border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-	
-	child = gtk_bin_get_child (GTK_BIN (widget));
-	
-	//state = gtk_widget_get_state_flags (widget);
-	context = gtk_widget_get_style_context (widget);
-	width = gtk_widget_get_allocated_width (widget);
-	height = gtk_widget_get_allocated_height (widget);
+	gint width = gtk_widget_get_allocated_width (widget);
 
+	gint x, y, w;
 	x = border_width;
 	y = border_width;
 	w = width - border_width * 2;
-	h = height - border_width * 2;
 
-	// draw the background
-	int n = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "gldi-step"));
-	if (n != 0)  // so we only draw the barkground if the item is or was selected
-	{
-		double a = (double)n/N;
-
-		cairo_save (cr);
-
-		int r=6, l=0;
-		cairo_dock_draw_rounded_rectangle (cr, r, l, w - 2*r, h);
-		cairo_clip (cr);
-
-		gldi_style_colors_set_selected_bg_color (cr);
-		cairo_paint_with_alpha (cr, a);
-
-		cairo_restore (cr);
-	}
+	// get the line color of the menu
+	GldiColor rgb;
+	if (myDialogsParam.bUseDefaultColors)
+		gldi_style_color_get (GLDI_COLOR_LINE, &rgb);
+	else
+		rgb = myDialogsParam.fLineColor;
 	
-	// draw the arrow in case of a sub-menu
-	if (gtk_menu_item_get_submenu (GTK_MENU_ITEM (widget)))
-	{
-		gint arrow_x, arrow_y;
-		gint arrow_size, spacing;
-		GtkTextDirection direction;
-		gdouble angle;
-
-		direction = gtk_widget_get_direction (widget);
-		get_arrow_size (widget, child, &arrow_size, &spacing);
-
-		if (direction == GTK_TEXT_DIR_LTR)
-		{
-		  arrow_x = x + w - arrow_size - spacing;
-		  angle = G_PI / 2;
-		}
-		else
-		{
-		  arrow_x = x + spacing;
-		  angle = (3 * G_PI) / 2;
-		}
-
-		arrow_y = y + (h - arrow_size) / 2;
-
-		gtk_render_arrow (context, cr, angle, arrow_x, arrow_y, arrow_size);
-	}
-	else if (!child)  // separator
-	{
-		gboolean wide_separators;
-		gint     separator_height;
-		GtkBorder padding;
-
-		gtk_style_context_get_padding (context, 0, &padding);
-		gtk_widget_style_get (widget,
-			"wide-separators",    &wide_separators,
-			"separator-height",   &separator_height,
-			NULL);
-		if (wide_separators)
-			gtk_render_frame (context, cr,
-				x + padding.left,
-				y + padding.top,
-				w - padding.left - padding.right,
-				separator_height);
-		else
-			gtk_render_line (context, cr,
-				x + padding.left,
-				y + padding.top,
-				x + w - padding.right - 1,
-				y + padding.top);
-	}
+	// make a pattern with the alpha channel: 0 - 0.1 ---- .9 - 1
+	int mb = w*.05;  // margin to border
+	cairo_pattern_t *pattern;
+	pattern = cairo_pattern_create_linear (x+mb, y, x+w-mb, y);
+	cairo_pattern_add_color_stop_rgba (pattern, 0., rgb.rgba.red, rgb.rgba.green, rgb.rgba.blue, rgb.rgba.alpha*.1);
+	cairo_pattern_add_color_stop_rgba (pattern, .1, rgb.rgba.red, rgb.rgba.green, rgb.rgba.blue, rgb.rgba.alpha);
+	cairo_pattern_add_color_stop_rgba (pattern, .9, rgb.rgba.red, rgb.rgba.green, rgb.rgba.blue, rgb.rgba.alpha);
+	cairo_pattern_add_color_stop_rgba (pattern, 1., rgb.rgba.red, rgb.rgba.green, rgb.rgba.blue, rgb.rgba.alpha*.1);
+	cairo_set_source (cr, pattern);
 	
-	// draw the item's content
-	GtkWidgetClass *parent_class = g_type_class_peek (g_type_parent (G_TYPE_FROM_INSTANCE (widget)));
-	parent_class = g_type_class_peek_parent (parent_class);
+	// draw the separator as a 1px line with a margin from the border
+	cairo_move_to(cr, x+mb, y);
+	cairo_set_line_width (cr, 1);
+	cairo_line_to(cr, x+w-mb, y);
+	cairo_stroke(cr);
+	cairo_pattern_destroy (pattern);
 	
-	cairo_set_source_rgba (cr, 1, 1, 1, 1);
-	
-	parent_class->draw (widget, cr);
 	return TRUE;  // intercept
 }
 
-static gboolean _update_menu_item (GtkWidget* pMenuItem, G_GNUC_UNUSED gpointer data)
-{
-	int n = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(pMenuItem), "gldi-step"));
-	gboolean bInside = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(pMenuItem), "gldi-inside"));
-	
-	if (bInside)
-	{
-		if (n < N)
-			n ++;
-	}
-	else
-	{
-		if (n > 0)
-			n --;
-	}
-	g_object_set_data (G_OBJECT(pMenuItem), "gldi-step", GINT_TO_POINTER (n));
-	
-	gtk_widget_queue_draw (pMenuItem);
-	
-	if (n == 0 || n == N)
-	{
-		guint iSidAnimation = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(pMenuItem), "gldi-animation"));
-		if (iSidAnimation != 0)
-		{
-			g_source_remove (iSidAnimation);
-			g_object_set_data (G_OBJECT(pMenuItem), "gldi-animation", NULL);
-		}
-		return FALSE;
-	}
-	return TRUE;
-}
-static gboolean _on_select_menu_item (GtkWidget* pMenuItem, G_GNUC_UNUSED gpointer data)
-{
-	guint iSidAnimation = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(pMenuItem), "gldi-animation"));
-	if (iSidAnimation != 0)
-		g_source_remove (iSidAnimation);
-	iSidAnimation = g_timeout_add (dt1, (GSourceFunc)_update_menu_item, pMenuItem);
-	g_object_set_data (G_OBJECT(pMenuItem), "gldi-animation", GUINT_TO_POINTER (iSidAnimation));
-	
-	g_object_set_data (G_OBJECT(pMenuItem), "gldi-inside", GINT_TO_POINTER (TRUE));
-	
-	return FALSE;
-}
-static gboolean _on_deselect_menu_item (GtkWidget* pMenuItem, G_GNUC_UNUSED gpointer data)
-{
-	guint iSidAnimation = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(pMenuItem), "gldi-animation"));
-	if (iSidAnimation != 0)
-		g_source_remove (iSidAnimation);
-	iSidAnimation = g_timeout_add (dt2, (GSourceFunc)_update_menu_item, pMenuItem);
-	g_object_set_data (G_OBJECT(pMenuItem), "gldi-animation", GUINT_TO_POINTER (iSidAnimation));
-	
-	g_object_set_data (G_OBJECT(pMenuItem), "gldi-inside", GINT_TO_POINTER (FALSE));
-	
-	return FALSE;
-}
-static void _on_destroy_menu_item (GtkWidget* pMenuItem, G_GNUC_UNUSED gpointer data)
-{
-	guint iSidAnimation = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(pMenuItem), "gldi-animation"));
-	if (iSidAnimation != 0)
-	{
-		g_source_remove (iSidAnimation);
-		g_object_set_data (G_OBJECT(pMenuItem), "gldi-animation", NULL);
-	}
-}
-*/
-
-GtkWidget *gldi_menu_item_new_full (const gchar *cLabel, const gchar *cImage, gboolean bUseMnemonic, GtkIconSize iSize)
+GtkWidget *gldi_menu_item_new_full2 (const gchar *cLabel, const gchar *cImage, gboolean bUseMnemonic, GtkIconSize iSize, gboolean bUseStyle)
 {
 	if (iSize == 0)
-		iSize = GTK_ICON_SIZE_MENU;
+		iSize = GTK_ICON_SIZE_MENU; // 16x16
 	
+	(void)cImage; // avoid warnings if CAIRO_DOCK_FORCE_ICON_IN_MENUS == 0
 	GtkWidget *pMenuItem;
-	if (! cImage)
+#if (CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1)
+	if (cImage)
 	{
-		if (! cLabel)
-			pMenuItem = gtk_menu_item_new ();
-		else
-			pMenuItem =  (bUseMnemonic ? gtk_menu_item_new_with_mnemonic (cLabel) : gtk_menu_item_new_with_label (cLabel));
-	}
-	else
-	{
-		GtkWidget *image = NULL;
-#if (! GTK_CHECK_VERSION (3, 10, 0)) || (CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1)
-		if (*cImage == '/')
-		{
-			int size;
-			gtk_icon_size_lookup (iSize, &size, NULL);
-			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (cImage, size, size, NULL);
-			image = gtk_image_new_from_pixbuf (pixbuf);
-			g_object_unref (pixbuf);
-		}
-		else if (*cImage != '\0')
-		{
-			image = gtk_image_new_from_icon_name (cImage, iSize);
-		}
-#endif
-		
-#if GTK_CHECK_VERSION (3, 10, 0)
-		#if (CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1)
 		if (! cLabel)
 			pMenuItem = gtk3_image_menu_item_new ();
 		else
 			pMenuItem = (bUseMnemonic ? gtk3_image_menu_item_new_with_mnemonic (cLabel) : gtk3_image_menu_item_new_with_label (cLabel));
-		gtk3_image_menu_item_set_image (GTK3_IMAGE_MENU_ITEM (pMenuItem), image);
-		#else
+		
+		if (*cImage)
+		{
+			GtkWidget *image = NULL;
+			int size;
+			gtk_icon_size_lookup (iSize, &size, NULL);
+			
+			// note: this takes care to load the icon with the correct scale factor
+			cairo_surface_t *surface = cairo_dock_create_surface_from_icon (cImage, size, size);
+			if (surface)
+			{
+				image = gtk_image_new_from_surface (surface);
+				gtk3_image_menu_item_set_image (GTK3_IMAGE_MENU_ITEM (pMenuItem), image);
+				cairo_surface_destroy (surface); // note: this does not destroy the surface, only unrefs it			
+			}
+		}
+	}
+	else
+#endif
+	{
 		if (! cLabel)
 			pMenuItem = gtk_menu_item_new ();
 		else
 			pMenuItem = (bUseMnemonic ? gtk_menu_item_new_with_mnemonic (cLabel) : gtk_menu_item_new_with_label (cLabel));
-		#endif
-#else
-		if (! cLabel)
-			pMenuItem = gtk_image_menu_item_new ();
-		else
-			pMenuItem = (bUseMnemonic ? gtk_image_menu_item_new_with_mnemonic (cLabel) : gtk_image_menu_item_new_with_label (cLabel));
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (pMenuItem), image);
-		#if (CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1)
-		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (pMenuItem), TRUE);
-		#endif
-#endif
 	}
 
-	_init_menu_item (pMenuItem);
+	if (bUseStyle) _init_menu_item (pMenuItem);
 
 	gtk_widget_show_all (pMenuItem);  // show immediately, so that the menu-item is realized when the menu is popped up
 	
@@ -971,29 +956,82 @@ GtkWidget *gldi_menu_item_new_full (const gchar *cLabel, const gchar *cImage, gb
 
 void gldi_menu_item_set_image (GtkWidget *pMenuItem, GtkWidget *image)
 {
-#if GTK_CHECK_VERSION (3, 10, 0)
-	#if (CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1)
+#if (CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1)
 	gtk3_image_menu_item_set_image (GTK3_IMAGE_MENU_ITEM (pMenuItem), image);
-	#else
-	g_object_unref (image);
-	#endif
 #else
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (pMenuItem), image);
-	#if (CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1)
-	gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (pMenuItem), TRUE);
-	#endif
+	(void)pMenuItem; // avoid warnings
+	g_object_ref_sink (image); // image will typically be newly allocated and have a floating reference
+	g_object_unref (image); // it is taken over here and freed in this case
 #endif
 }
 
 GtkWidget *gldi_menu_item_get_image (GtkWidget *pMenuItem)
 {
-	#if GTK_CHECK_VERSION (3, 10, 0)
 	return gtk3_image_menu_item_get_image (GTK3_IMAGE_MENU_ITEM (pMenuItem));
-	#else
-	return gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (pMenuItem));
-	#endif
 }
 
+
+// custom data to store with menu items that have a tooltip
+
+typedef struct _GldiMenuItemTooltip {
+	gchar *cText;
+	void (*pFunction)(GtkMenuItem*, gpointer);
+	gpointer pData;
+} GldiMenuItemTooltip;
+
+static void _free_tooltip_data (gpointer ptr, GObject*)
+{
+	if (!ptr) return;
+	GldiMenuItemTooltip *pCustomData = (GldiMenuItemTooltip*)ptr;
+	g_free (pCustomData->cText);
+	g_free (pCustomData);
+	// note: pData is not managed by us
+}
+
+static void _enable_tooltip (GtkWidget* pWidget, gpointer ptr)
+{
+	if (!ptr) return; // should not happen
+	GldiMenuItemTooltip *pCustomData = (GldiMenuItemTooltip*)ptr;
+	gtk_widget_set_tooltip_text (pWidget, pCustomData->cText);
+}
+
+static void _tooltip_activate (GtkMenuItem* pMenuItem, gpointer ptr)
+{
+	gtk_widget_set_tooltip_text (GTK_WIDGET (pMenuItem), NULL);
+	if (!ptr) return; // should not happen
+	GldiMenuItemTooltip *pCustomData = (GldiMenuItemTooltip*)ptr;
+	if (pCustomData->pFunction)
+		(pCustomData->pFunction)(pMenuItem, pCustomData->pData);
+}
+
+static GtkWidget *_menu_item_new_with_action_and_tooltip (const gchar *cLabel, const gchar *cImage, const gchar *cToolTip, void (*pFunction)(GtkMenuItem*, gpointer), gpointer pData)
+{
+	GtkWidget *pMenuItem;
+	gboolean bCustomCallback = ((cToolTip != NULL) && gldi_wayland_manager_have_layer_shell ());
+	
+	if (bCustomCallback)
+	{
+		pMenuItem = gldi_menu_item_new (cLabel, cImage);
+		if (!pMenuItem) return NULL;
+		
+		GldiMenuItemTooltip *pCustomData = g_new0 (GldiMenuItemTooltip, 1);
+		pCustomData->cText = g_strdup (cToolTip);
+		pCustomData->pFunction = pFunction;
+		pCustomData->pData = pData;
+		
+		g_object_weak_ref (G_OBJECT (pMenuItem), _free_tooltip_data, pCustomData);
+		g_signal_connect (G_OBJECT (pMenuItem), "map", G_CALLBACK (_enable_tooltip), pCustomData);
+		g_signal_connect (G_OBJECT (pMenuItem), "activate", G_CALLBACK (_tooltip_activate), pCustomData);
+	}
+	else
+	{
+		pMenuItem = gldi_menu_item_new_with_action (cLabel, cImage, G_CALLBACK (pFunction), pData);
+		if (pMenuItem && cToolTip) gtk_widget_set_tooltip_text (pMenuItem, cToolTip);
+	}
+	return pMenuItem;
+}
+
+
 GtkWidget *gldi_menu_item_new_with_action (const gchar *cLabel, const gchar *cImage, GCallback pFunction, gpointer pData)
 {
 	GtkWidget *pMenuItem = gldi_menu_item_new (cLabel, cImage);
@@ -1006,7 +1044,7 @@ GtkWidget *gldi_menu_item_new_with_submenu (const gchar *cLabel, const gchar *cI
 {
 	GtkIconSize iSize;
 	if (cImage && (*cImage == '/' || *cImage == '\0'))  // for icons that are not stock-icons, we choose a bigger size; the reason is that these icons usually don't have a 16x16 version, and don't scale very well to such a small size (most of the time, it's the icon of an application, or the cairo-dock or recent-documents icon (note: for these 2, we could make a small version)). it's a workaround and a better solution may exist ^^
-		iSize = GTK_ICON_SIZE_LARGE_TOOLBAR;
+		iSize = GTK_ICON_SIZE_LARGE_TOOLBAR; // 24x24
 	else
 		iSize = 0;
 	GtkWidget *pMenuItem = gldi_menu_item_new_full (cLabel, cImage, FALSE, iSize);
@@ -1024,6 +1062,13 @@ GtkWidget *gldi_menu_add_item (GtkWidget *pMenu, const gchar *cLabel, const gcha
 	return pMenuItem;
 }
 
+GtkWidget *gldi_menu_add_item_with_tooltip (GtkWidget *pMenu, const gchar *cLabel, const gchar *cImage, const gchar *cToolTip, void (*pFunction)(GtkMenuItem*, gpointer), gpointer pData)
+{
+	GtkWidget *pMenuItem = _menu_item_new_with_action_and_tooltip (cLabel, cImage, cToolTip, pFunction, pData);
+	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
+	return pMenuItem;
+}
+
 GtkWidget *gldi_menu_add_sub_menu_full (GtkWidget *pMenu, const gchar *cLabel, const gchar *cImage, GtkWidget **pMenuItemPtr)
 {
 	GtkWidget *pSubMenu;
@@ -1043,9 +1088,6 @@ void gldi_menu_add_separator (GtkWidget *pMenu)
 
 gboolean GLDI_IS_IMAGE_MENU_ITEM (GtkWidget *pMenuItem)  // defined as a function to not export gtk3imagemenuitem.h
 {
-	#if GTK_CHECK_VERSION (3, 10, 0)
 	return GTK3_IS_IMAGE_MENU_ITEM (pMenuItem);
-	#else
-	return GTK_IS_IMAGE_MENU_ITEM (pMenuItem);
-	#endif
 }
+
