@@ -17,26 +17,38 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#define _GNU_SOURCE
 #include <string.h>
 #include <math.h>
 #include <pango/pango.h>
+#include <fcntl.h>
+#include <stdio.h>
 
 #include "cairo-dock-log.h"
 #include "cairo-dock-draw.h"
 #include "cairo-dock-launcher-manager.h"
 #include "cairo-dock-container.h"
 #include "cairo-dock-image-buffer.h"
-#include "cairo-dock-X-manager.h"
+#include "cairo-dock-desktop-manager.h"
 #include "cairo-dock-icon-manager.h"  // cairo_dock_search_icon_s_path
+#include "cairo-dock-dialog-manager.h"
+#include "cairo-dock-style-manager.h"
 #include "cairo-dock-surface-factory.h"
 
-extern CairoContainer *g_pPrimaryContainer;
+extern GldiContainer *g_pPrimaryContainer;
 extern gboolean g_bUseOpenGL;
 
-static cairo_t *s_pSourceContext = NULL;
 
-
-void cairo_dock_calculate_size_fill (double *fImageWidth, double *fImageHeight, int iWidthConstraint, int iHeightConstraint, gboolean bNoZoomUp, double *fZoomWidth, double *fZoomHeight)
+/* Calcule la taille d'une image selon une contrainte en largeur et hauteur de manière à remplir l'espace donné.
+*@param fImageWidth the width of the image. Contient initialement the width of the image, et sera écrasée avec la largeur obtenue.
+*@param fImageHeight the height of the image. Contient initialement the height of the image, et sera écrasée avec la hauteur obtenue.
+*@param iWidthConstraint contrainte en largeur (0 <=> pas de contrainte).
+*@param iHeightConstraint contrainte en hauteur (0 <=> pas de contrainte).
+*@param bNoZoomUp TRUE ssi on ne doit pas agrandir the image (seulement la rétrécir).
+*@param fZoomWidth sera renseigné avec le facteur de zoom en largeur qui a été appliqué.
+*@param fZoomHeight sera renseigné avec le facteur de zoom en hauteur qui a été appliqué.
+*/
+static void _cairo_dock_calculate_size_fill (double *fImageWidth, double *fImageHeight, int iWidthConstraint, int iHeightConstraint, gboolean bNoZoomUp, double *fZoomWidth, double *fZoomHeight)
 {
 	if (iWidthConstraint != 0)
 	{
@@ -60,7 +72,15 @@ void cairo_dock_calculate_size_fill (double *fImageWidth, double *fImageHeight,
 		*fZoomHeight = 1.;
 }
 
-void cairo_dock_calculate_size_constant_ratio (double *fImageWidth, double *fImageHeight, int iWidthConstraint, int iHeightConstraint, gboolean bNoZoomUp, double *fZoom)
+/* Calcule la taille d'une image selon une contrainte en largeur et hauteur en gardant le ratio hauteur/largeur constant.
+*@param fImageWidth the width of the image. Contient initialement the width of the image, et sera écrasée avec la largeur obtenue.
+*@param fImageHeight the height of the image. Contient initialement the height of the image, et sera écrasée avec la hauteur obtenue.
+*@param iWidthConstraint contrainte en largeur (0 <=> pas de contrainte).
+*@param iHeightConstraint contrainte en hauteur (0 <=> pas de contrainte).
+*@param bNoZoomUp TRUE ssi on ne doit pas agrandir the image (seulement la rétrécir).
+*@param fZoom sera renseigné avec le facteur de zoom qui a été appliqué.
+*/
+static void _cairo_dock_calculate_size_constant_ratio (double *fImageWidth, double *fImageHeight, int iWidthConstraint, int iHeightConstraint, gboolean bNoZoomUp, double *fZoom)
 {
 	if (iWidthConstraint != 0 && iHeightConstraint != 0)
 		*fZoom = MIN (iWidthConstraint / (*fImageWidth), iHeightConstraint / (*fImageHeight));
@@ -77,8 +97,16 @@ void cairo_dock_calculate_size_constant_ratio (double *fImageWidth, double *fIma
 }
 
 
-
-void cairo_dock_calculate_constrainted_size (double *fImageWidth, double *fImageHeight, int iWidthConstraint, int iHeightConstraint, CairoDockLoadImageModifier iLoadingModifier, double *fZoomWidth, double *fZoomHeight)
+/* Calculate the size of an image according to a constraint on width and height, and a loading modifier.
+*@param fImageWidth pointer to the width of the image. Initially contains the width of the original image, and is updated with the resulting width.
+*@param fImageHeight pointer to the height of the image. Initially contains the height of the original image, and is updated with the resulting height.
+*@param iWidthConstraint constraint on width (0 <=> no constraint).
+*@param iHeightConstraint constraint on height (0 <=> no constraint).
+*@param iLoadingModifier a mask of different loading modifiers.
+*@param fZoomWidth will be filled with the zoom that has been applied on width.
+*@param fZoomHeight will be filled with the zoom that has been applied on height.
+*/
+static void _cairo_dock_calculate_constrainted_size (double *fImageWidth, double *fImageHeight, int iWidthConstraint, int iHeightConstraint, CairoDockLoadImageModifier iLoadingModifier, double *fZoomWidth, double *fZoomHeight)
 {
 	gboolean bFillSpace = iLoadingModifier & CAIRO_DOCK_FILL_SPACE;
 	gboolean bKeepRatio = iLoadingModifier & CAIRO_DOCK_KEEP_RATIO;
@@ -120,7 +148,7 @@ void cairo_dock_calculate_constrainted_size (double *fImageWidth, double *fImage
 	
 	if (bKeepRatio)
 	{
-		cairo_dock_calculate_size_constant_ratio (fImageWidth,
+		_cairo_dock_calculate_size_constant_ratio (fImageWidth,
 			fImageHeight,
 			iWidthConstraint,
 			iHeightConstraint,
@@ -139,7 +167,7 @@ void cairo_dock_calculate_constrainted_size (double *fImageWidth, double *fImage
 	}
 	else
 	{
-		cairo_dock_calculate_size_fill (fImageWidth,
+		_cairo_dock_calculate_size_fill (fImageWidth,
 			fImageHeight,
 			iWidthConstraint,
 			iHeightConstraint,
@@ -150,34 +178,48 @@ void cairo_dock_calculate_constrainted_size (double *fImageWidth, double *fImage
 }
 
 
-#define _get_source_context(...) \
-	__extension__ ({\
-	if (s_pSourceContext == NULL) {\
-		if (g_pPrimaryContainer != NULL)\
-			s_pSourceContext = cairo_dock_create_drawing_context_generic (g_pPrimaryContainer); }\
-	s_pSourceContext; })
-#define _destroy_source_context(...) do {\
-	if (s_pSourceContext != NULL) {\
-		cairo_destroy (s_pSourceContext);\
-		s_pSourceContext = NULL; } } while (0)
-
-void cairo_dock_reset_source_context (void)
+static inline cairo_t *_get_source_context (void)
 {
-	_destroy_source_context ();
+	cairo_t *pSourceContext = NULL;
+	if (g_pPrimaryContainer != NULL)
+	{
+		gtk_widget_realize (g_pPrimaryContainer->pWidget);  // ensure the widget is realized
+		// this is deprecated, but it is a convenient way to get a cairo context whose properties we can use when creating our surfaces
+		pSourceContext = gdk_cairo_create (gldi_container_get_gdk_window(g_pPrimaryContainer));
+	}
+	return pSourceContext;  // Note: we can't keep the context alive and reuse it later, because under Wayland it will make the container invisible
 }
 
-cairo_surface_t *cairo_dock_create_blank_surface (int iWidth, int iHeight)
+cairo_surface_t *cairo_dock_create_blank_surface_full (int iWidth, int iHeight, cairo_t *pSourceContext)
 {
-	cairo_t *pSourceContext = _get_source_context ();
-	if (pSourceContext != NULL && cairo_status (pSourceContext) == CAIRO_STATUS_SUCCESS && ! g_bUseOpenGL)
-		return cairo_surface_create_similar (cairo_get_target (pSourceContext),
+	cairo_t *tmpContext = NULL;
+	if (!pSourceContext && !g_bUseOpenGL)
+		pSourceContext = tmpContext = _get_source_context ();
+	cairo_surface_t *pSurface;
+	if (pSourceContext != NULL && cairo_status (pSourceContext) != CAIRO_STATUS_SUCCESS)
+		pSourceContext = NULL;
+	if (g_bUseOpenGL || pSourceContext == NULL)
+	{
+		// opengl or invalid context -> create an image that is a mere ARGB buffer that can be mapped into a texture
+		double xs = 1.0, ys = 1.0; // take into account the source context scale
+		if (pSourceContext != NULL)
+			cairo_surface_get_device_scale (cairo_get_target (pSourceContext), &xs, &ys);
+		else if (g_pPrimaryContainer != NULL)
+		{
+			GdkWindow* gdkwindow = gldi_container_get_gdk_window (g_pPrimaryContainer);
+			xs = ys = gdk_window_get_scale_factor (gdkwindow);
+		}
+		pSurface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, (int)ceil(iWidth * xs), (int)ceil(iHeight * ys));
+		cairo_surface_set_device_scale (pSurface, xs, ys);
+	}
+	else
+		pSurface = cairo_surface_create_similar (cairo_get_target (pSourceContext),
 			CAIRO_CONTENT_COLOR_ALPHA,
 			iWidth,
 			iHeight);
-	else
-		return cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-			iWidth,
-			iHeight);
+		
+	if(tmpContext) cairo_destroy (tmpContext);
+	return pSurface;
 }
 
 static inline void _apply_orientation_and_scale (cairo_t *pCairoContext, CairoDockLoadImageModifier iLoadingModifier, double fImageWidth, double fImageHeight, double fZoomX, double fZoomY, double fUsefulWidth, double fUsefulheight)
@@ -293,7 +335,7 @@ cairo_surface_t *cairo_dock_create_surface_from_xicon_buffer (gulong *pXIconBuff
 	
 	double fWidth = w, fHeight = h;
 	double fIconWidthSaturationFactor = 1., fIconHeightSaturationFactor = 1.;
-	cairo_dock_calculate_constrainted_size (&fWidth,
+	_cairo_dock_calculate_constrainted_size (&fWidth,
 		&fHeight,
 		iWidth,
 		iHeight,
@@ -328,7 +370,7 @@ cairo_surface_t *cairo_dock_create_surface_from_pixbuf (GdkPixbuf *pixbuf, doubl
 	*fImageWidth = gdk_pixbuf_get_width (pixbuf);
 	*fImageHeight = gdk_pixbuf_get_height (pixbuf);
 	double fIconWidthSaturationFactor = 1., fIconHeightSaturationFactor = 1.;
-	cairo_dock_calculate_constrainted_size (fImageWidth,
+	_cairo_dock_calculate_constrainted_size (fImageWidth,
 			fImageHeight,
 			iWidthConstraint,
 			iHeightConstraint,
@@ -403,6 +445,72 @@ cairo_surface_t *cairo_dock_create_surface_from_pixbuf (GdkPixbuf *pixbuf, doubl
 }
 
 
+GdkPixbuf *cairo_dock_load_gdk_pixbuf (const gchar *cImagePath, int iWidth, int iHeight)
+{
+	GError *erreur = NULL;
+	// note: g_file_read sets O_CLOEXEC by default, but gdk_pixbuf_new_from_file () does not
+	GFile *file = g_file_new_for_path (cImagePath);
+	GInputStream *istream = G_INPUT_STREAM (g_file_read (file, NULL, &erreur));
+	GdkPixbuf *pixbuf = NULL;
+	if (istream)
+	{
+		// type is detected automatically
+		if (iWidth >= 0 || iHeight >= 0)
+			pixbuf = gdk_pixbuf_new_from_stream_at_scale (istream, iWidth, iHeight, TRUE, NULL, &erreur);
+		else pixbuf = gdk_pixbuf_new_from_stream (istream, NULL, &erreur);
+		g_object_unref (istream);
+	}
+	g_object_unref (file);
+	
+	if (erreur != NULL)
+	{
+		cd_warning (erreur->message);
+		g_error_free (erreur);
+		return NULL;
+	}
+	return pixbuf;
+}
+
+GdkPixbuf *cairo_dock_load_gdk_pixbuf_with_max_size (const gchar *cImagePath, int iMaxWidth, int iMaxHeight)
+{
+	GdkPixbuf *pPixbuf = cairo_dock_load_gdk_pixbuf (cImagePath, -1, -1);
+	if (!pPixbuf) return NULL;
+	
+	int w = gdk_pixbuf_get_width (pPixbuf);
+	int h = gdk_pixbuf_get_height (pPixbuf);
+	gboolean bScale = FALSE;
+	
+	if (iMaxWidth > 0 && w > iMaxWidth)
+	{
+		h *= ((double)iMaxWidth)/w;
+		w = iMaxWidth;
+		bScale = TRUE;
+	}
+	if (iMaxHeight > 0 && h > iMaxHeight)
+	{
+		w *= ((double)iMaxHeight)/h;
+		h = iMaxHeight;
+		bScale = TRUE;
+	}
+	
+	if (bScale)
+	{
+		GdkPixbuf *tmp = pPixbuf;
+		pPixbuf = gdk_pixbuf_scale_simple (tmp, w, h, GDK_INTERP_BILINEAR);
+		g_object_unref (tmp);
+	}
+	
+	return pPixbuf;
+}
+
+static cairo_status_t _cairo_read_func (void *ptr, unsigned char *data, unsigned int length)
+{
+	FILE *f = (FILE*)ptr;
+	size_t res = fread (data, 1, length, f);
+	if (res == (size_t)length) return CAIRO_STATUS_SUCCESS;
+	return CAIRO_STATUS_READ_ERROR;
+}
+
 cairo_surface_t *cairo_dock_create_surface_from_image (const gchar *cImagePath, double fMaxScale, int iWidthConstraint, int iHeightConstraint, CairoDockLoadImageModifier iLoadingModifier, double *fImageWidth, double *fImageHeight, double *fZoomX, double *fZoomY)
 {
 	//g_print ("%s (%s, %dx%dx%.2f, %d)\n", __func__, cImagePath, iWidthConstraint, iHeightConstraint, fMaxScale, iLoadingModifier);
@@ -410,19 +518,20 @@ cairo_surface_t *cairo_dock_create_surface_from_image (const gchar *cImagePath,
 	GError *erreur = NULL;
 	RsvgDimensionData rsvg_dimension_data;
 	RsvgHandle *rsvg_handle = NULL;
-	cairo_surface_t* surface_ini;
+	cairo_surface_t* surface_ini = NULL;
 	cairo_surface_t* pNewSurface = NULL;
 	cairo_t* pCairoContext = NULL;
 	double fIconWidthSaturationFactor = 1.;
 	double fIconHeightSaturationFactor = 1.;
 	
 	//\_______________ On cherche a determiner le type de l'image. En effet, les SVG et les PNG sont charges differemment des autres.
-	gboolean bIsSVG = FALSE, bIsPNG = FALSE, bIsXPM = FALSE;
-	FILE *fd = fopen (cImagePath, "r");
-	if (fd != NULL)
+	gboolean bIsSVG = FALSE, bIsPNG = FALSE, bIsXPM = FALSE, bIsResource = FALSE;
+	int fd = open (cImagePath, O_RDONLY | O_CLOEXEC);
+	if (fd >= 0)
 	{
+		FILE *tmpf = fdopen (fd, "r");
 		char buffer[8];
-		if (fgets (buffer, 7, fd) != NULL)
+		if (fgets (buffer, 7, tmpf) != NULL)
 		{
 			if (strncmp (buffer+2, "xml", 3) == 0)
 				bIsSVG = TRUE;
@@ -432,14 +541,24 @@ cairo_surface_t *cairo_dock_create_surface_from_image (const gchar *cImagePath,
 				bIsXPM = TRUE;
 			//cd_debug ("  format : %d;%d;%d", bIsSVG, bIsPNG, bIsXPM);
 		}
-		fclose (fd);
+		fclose (tmpf); // will close fd as well
 	}
 	else
 	{
-		cd_warning ("This file (%s) doesn't exist or is not readable.", cImagePath);
-		return NULL;
+		if (strncmp (cImagePath, "/org/gtk", 8) == 0)
+		{
+			// This is a GDK / GTK resource (typically for a stock icon), but GTK "forgot"
+			// to include the resource:// prefix in the filename; see e.g.
+			// https://gitlab.gnome.org/GNOME/gtk/-/issues/4983
+			bIsResource = TRUE;
+		}
+		else
+		{
+			cd_warning ("This file (%s) doesn't exist or is not readable.", cImagePath);
+			return NULL;
+		}
 	}
-	if (! bIsSVG && ! bIsPNG && ! bIsXPM)  // sinon en desespoir de cause on se base sur l'extension.
+	if (! bIsSVG && ! bIsPNG && ! bIsXPM && ! bIsResource)  // sinon en desespoir de cause on se base sur l'extension.
 	{
 		//cd_debug ("  on se base sur l'extension en desespoir de cause.");
 		if (g_str_has_suffix (cImagePath, ".svg"))
@@ -467,7 +586,7 @@ cairo_surface_t *cairo_dock_create_surface_from_image (const gchar *cImagePath,
 			*fImageWidth = (gdouble) w;
 			*fImageHeight = (gdouble) h;
 			//g_print ("%.2fx%.2f\n", *fImageWidth, *fImageHeight);
-			cairo_dock_calculate_constrainted_size (fImageWidth,
+			_cairo_dock_calculate_constrainted_size (fImageWidth,
 				fImageHeight,
 				iWidthConstraint,
 				iHeightConstraint,
@@ -495,14 +614,18 @@ cairo_surface_t *cairo_dock_create_surface_from_image (const gchar *cImagePath,
 	}
 	else if (bIsPNG)
 	{
-		surface_ini = cairo_image_surface_create_from_png (cImagePath);  // cree un fond noir :-(
-		if (cairo_surface_status (surface_ini) == CAIRO_STATUS_SUCCESS)
+		int fd = open (cImagePath, O_RDONLY | O_CLOEXEC);
+		FILE *file = (fd >= 0) ? fdopen (fd, "r") : NULL;
+		if (file) surface_ini = cairo_image_surface_create_from_png_stream (
+			_cairo_read_func, (void*)file);
+		if (file) fclose (file); // will close fd as well
+		if (surface_ini && cairo_surface_status (surface_ini) == CAIRO_STATUS_SUCCESS)
 		{
 			int w = cairo_image_surface_get_width (surface_ini);
 			int h = cairo_image_surface_get_height (surface_ini);
 			*fImageWidth = (double) w;
 			*fImageHeight = (double) h;
-			cairo_dock_calculate_constrainted_size (fImageWidth,
+			_cairo_dock_calculate_constrainted_size (fImageWidth,
 				fImageHeight,
 				iWidthConstraint,
 				iHeightConstraint,
@@ -531,17 +654,22 @@ cairo_surface_t *cairo_dock_create_surface_from_image (const gchar *cImagePath,
 			cairo_paint (pCairoContext);
 			cairo_destroy (pCairoContext);
 		}
-		cairo_surface_destroy (surface_ini);
+		if (surface_ini) cairo_surface_destroy (surface_ini);
 	}
 	else  // le code suivant permet de charger tout type d'image, mais en fait c'est un peu idiot d'utiliser des icones n'ayant pas de transparence.
 	{
-		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (cImagePath, &erreur);  // semble se baser sur l'extension pour definir le type !
-		if (erreur != NULL)
+		GdkPixbuf *pixbuf = NULL;
+		if (bIsResource)
 		{
-			cd_warning (erreur->message);
-			g_error_free (erreur);
-			return NULL;
+			pixbuf = gdk_pixbuf_new_from_resource (cImagePath, &erreur);
+			if (erreur) {
+				cd_warning (erreur->message);
+				g_error_free (erreur);
+			}
 		}
+		else pixbuf = cairo_dock_load_gdk_pixbuf (cImagePath, -1, -1);
+		if (! pixbuf) return NULL; // warning message already printed above
+
 		pNewSurface = cairo_dock_create_surface_from_pixbuf (pixbuf,
 			fMaxScale,
 			iWidthConstraint,
@@ -703,269 +831,80 @@ cairo_surface_t * cairo_dock_rotate_surface (cairo_surface_t *pSurface, double f
 }
 
 
-/**static cairo_surface_t * cairo_dock_create_reflection_surface_horizontal (cairo_surface_t *pSurface, double fImageWidth, double fImageHeight, double fReflectSize, double fAlbedo, gboolean bDirectionUp)
+cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, GldiTextDescription *pTextDescription, double fMaxScale, int iMaxWidth, int *iTextWidth, int *iTextHeight)
 {
-	//g_print ("%s (%d)\n", __func__, bDirectionUp);
-
-	//\_______________ On cree la surface d'une fraction hauteur de l'image originale.
-	if (pSurface == NULL || fReflectSize == 0 || fAlbedo == 0)
-		return NULL;
-	cairo_surface_t *pNewSurface = cairo_dock_create_blank_surface (
-		fImageWidth,
-		fReflectSize);
-	cairo_t *pCairoContext = cairo_create (pNewSurface);
-	
-	cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
-	cairo_set_source_rgba (pCairoContext, 0., 0., 0., 0.);
-	
-	//\_______________ On dessine l'image originale inversee.
-	cairo_translate (pCairoContext, 0, fImageHeight);
-	cairo_scale (pCairoContext, 1., -1.);
-	cairo_set_source_surface (pCairoContext, pSurface, 0, (bDirectionUp ? 0 : fImageHeight - fReflectSize));
-	
-	//\_______________ On applique un degrade en transparence.
-	cairo_pattern_t *pGradationPattern = cairo_pattern_create_linear (0.,
-		fImageHeight,
-		0.,
-		fImageHeight - fReflectSize);  // de haut en bas.
-	g_return_val_if_fail (cairo_pattern_status (pGradationPattern) == CAIRO_STATUS_SUCCESS, NULL);
-
-	cairo_pattern_set_extend (pGradationPattern, CAIRO_EXTEND_NONE);
-	cairo_pattern_add_color_stop_rgba (pGradationPattern,
-		0.,
-		0.,
-		0.,
-		0.,
-		(bDirectionUp ? fAlbedo : 0.));
-	cairo_pattern_add_color_stop_rgba (pGradationPattern,
-		1.,
-		0.,
-		0.,
-		0.,
-		(bDirectionUp ? 0 : fAlbedo));
-
-	cairo_mask (pCairoContext, pGradationPattern);
-
-	cairo_pattern_destroy (pGradationPattern);
-	cairo_destroy (pCairoContext);
-	return pNewSurface;
-}
-
-static cairo_surface_t * cairo_dock_create_reflection_surface_vertical (cairo_surface_t *pSurface, double fImageWidth, double fImageHeight, double fReflectSize, double fAlbedo, gboolean bDirectionUp)
-{
-	g_return_val_if_fail (pSurface != NULL, NULL);
-
-	//\_______________ On cree la surface d'une fraction hauteur de l'image originale.
-	if (fReflectSize == 0 || fAlbedo == 0)
-		return NULL;
-	cairo_surface_t *pNewSurface = cairo_dock_create_blank_surface (
-		fReflectSize,
-		fImageHeight);
-	cairo_t *pCairoContext = cairo_create (pNewSurface);
-
-	cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
-	cairo_set_source_rgba (pCairoContext, 0., 0., 0., 0.);
-	
-	//\_______________ On dessine l'image originale inversee.
-	cairo_translate (pCairoContext, fImageWidth, 0);
-	cairo_scale (pCairoContext, -1., 1.);
-	cairo_set_source_surface (pCairoContext, pSurface, (bDirectionUp ? 0. : fImageHeight - fReflectSize), 0.);
-	
-	//\_______________ On applique un degrade en transparence.
-	cairo_pattern_t *pGradationPattern = cairo_pattern_create_linear (0,
-		0.,
-		fImageHeight - fReflectSize,
-		0.);  // de gauche a droite.
-	g_return_val_if_fail (cairo_pattern_status (pGradationPattern) == CAIRO_STATUS_SUCCESS, NULL);
-
-	cairo_pattern_set_extend (pGradationPattern, CAIRO_EXTEND_REPEAT);
-	cairo_pattern_add_color_stop_rgba (pGradationPattern,
-		0.,
-		0.,
-		0.,
-		0.,
-		(bDirectionUp ? fAlbedo : 0.));
-	cairo_pattern_add_color_stop_rgba (pGradationPattern,
-		1.,
-		0.,
-		0.,
-		0.,
-		(bDirectionUp ? 0. : fAlbedo));
-
-	cairo_mask (pCairoContext, pGradationPattern);
-
-	cairo_pattern_destroy (pGradationPattern);
-	cairo_destroy (pCairoContext);
-	return pNewSurface;
-}
-
-cairo_surface_t * cairo_dock_create_reflection_surface (cairo_surface_t *pSurface, double fImageWidth, double fImageHeight, double fReflectSize, double fAlbedo, gboolean bIsHorizontal, gboolean bDirectionUp)
-{
-	if (bIsHorizontal)
-		return cairo_dock_create_reflection_surface_horizontal (pSurface, fImageWidth, fImageHeight, fReflectSize, fAlbedo, bDirectionUp);
-	else
-		return cairo_dock_create_reflection_surface_vertical (pSurface, fImageWidth, fImageHeight, fReflectSize, fAlbedo, bDirectionUp);
-}*/
-
-
-static void _cairo_dock_limit_string_width (gchar *cLine, PangoLayout *pLayout, gboolean bUseMarkup, int iMaxWidth)
-{
-	//g_print ("%s (%s)\n", __func__, cLine);
-	// on insere des retours chariot pour tenir dans la largeur donnee.
-	PangoRectangle log;
-	gchar *sp, *last_sp=NULL;
-	double w;
-	int iNbLines = 0;
-	
-	gchar *str = cLine;
-	while (*str == ' ')  // on saute les espaces en debut de ligne.
-		str ++;
-	
-	sp = str;
-	do
-	{
-		sp = strchr (sp+1, ' ');  // on trouve l'espace suivant.
-		if (!sp)  // plus d'espace, on quitte.
-			break ;
-		
-		*sp = '\0';  // on coupe a cet espace.
-		if (bUseMarkup)  // on regarde la taille de str a sp.
-			pango_layout_set_markup (pLayout, str, -1);
-		else
-			pango_layout_set_text (pLayout, str, -1);
-		pango_layout_get_pixel_extents (pLayout, NULL, &log);
-		//g_print ("%s => w:%d, x:%d\n", str, log.width, log.x);
-		w = log.width + log.x;
-		
-		if (w > iMaxWidth)  // on deborde.
-		{
-			if (last_sp != NULL)  // on coupe au dernier espace connu.
-			{
-				*sp = ' ';  // on remet l'espace.
-				*last_sp = '\n';  // on coupe.
-				iNbLines ++;
-				str = last_sp + 1;  // on place le debut de ligne apres la coupure.
-			}
-			else  // aucun espace, c'est un mot entier.
-			{
-				*sp = '\n';  // on coupe apres le mot.
-				iNbLines ++;
-				str = sp + 1;  // on place le debut de ligne apres la coupure.
-			}
-			
-			while (*str == ' ')  // on saute les espaces en debut de ligne.
-				str ++;
-			sp = str;
-			last_sp = NULL;
-		}
-		else  // ca rentre.
-		{
-			*sp = ' ';  // on remet l'espace.
-			last_sp = sp;  // on memorise la derniere cesure qui fait tenir la ligne en largeur.
-			sp ++;  // on se place apres.
-			while (*sp == ' ')  // on saute tous les espaces.
-				sp ++;
-		}
-	} while (sp);
-	
-	// dernier mot.
-	if (bUseMarkup)  // on regarde la taille de str a sp.
-		pango_layout_set_markup (pLayout, str, -1);
-	else
-		pango_layout_set_text (pLayout, str, -1);
-	pango_layout_get_pixel_extents (pLayout, NULL, &log);
-	w = log.width + log.x;
-	if (w > iMaxWidth)  // on deborde.
-	{
-		if (last_sp != NULL)  // on coupe au dernier espace connu.
-			*last_sp = '\n';
-	}
-}
-
-cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, CairoDockLabelDescription *pLabelDescription, double fMaxScale, int iMaxWidth, int *iTextWidth, int *iTextHeight)
-{
-	g_return_val_if_fail (cText != NULL && pLabelDescription != NULL, NULL);
+	g_return_val_if_fail (cText != NULL && pTextDescription != NULL, NULL);
 	cairo_t *pSourceContext = _get_source_context ();
 	g_return_val_if_fail (pSourceContext != NULL && cairo_status (pSourceContext) == CAIRO_STATUS_SUCCESS, NULL);
 	
-	//\_________________ On ecrit le texte dans un calque Pango.
+	//\_________________ get the font description
+	PangoFontDescription *pDesc = gldi_text_description_get_description (pTextDescription);
+	if (!pDesc)
+		cd_debug ("no text desc for '%s'", cText);
+	int iSize = gldi_text_description_get_size (pTextDescription);
+	pango_font_description_set_absolute_size (pDesc, fMaxScale * iSize * PANGO_SCALE);
+	
+	//\_________________ create a layout
 	PangoLayout *pLayout = pango_cairo_create_layout (pSourceContext);
-	PangoRectangle log;
-	
-	PangoFontDescription *pDesc = pango_font_description_new ();
-	pango_font_description_set_absolute_size (pDesc, fMaxScale * pLabelDescription->iSize * PANGO_SCALE);
-	pango_font_description_set_family_static (pDesc, pLabelDescription->cFont);
-	pango_font_description_set_weight (pDesc, pLabelDescription->iWeight);
-	pango_font_description_set_style (pDesc, pLabelDescription->iStyle);
 	pango_layout_set_font_description (pLayout, pDesc);
-	pango_font_description_free (pDesc);
 	
-	if (pLabelDescription->bUseMarkup)
+	if (pTextDescription->bUseMarkup)
 		pango_layout_set_markup (pLayout, cText, -1);
 	else
 		pango_layout_set_text (pLayout, cText, -1);
 	
-	//\_________________ On insere des retours chariot si necessaire.
+	//\_________________ handle max width
+	if (pTextDescription->fMaxRelativeWidth != 0)
+	{
+		int iMaxLineWidth = pTextDescription->fMaxRelativeWidth * gldi_desktop_get_width() / g_desktopGeometry.iNbScreens;  // use the mean screen width since the text might be placed anywhere on the X screen.
+		pango_layout_set_width (pLayout, iMaxLineWidth * PANGO_SCALE);  // PANGO_WRAP_WORD by default
+	}
+	PangoRectangle log;
 	pango_layout_get_pixel_extents (pLayout, NULL, &log);
 	
-	if (pLabelDescription->fMaxRelativeWidth != 0)
-	{
-		int iMaxLineWidth = pLabelDescription->fMaxRelativeWidth * gldi_get_desktop_width() / g_desktopGeometry.iNbScreens;  // use the mean screen width since the text might be placed anywhere on the X screen.
-		int w = log.width;
-		//g_print ("text width : %d / %d\n", w, iMaxLineWidth);
-		if (w > iMaxLineWidth)  // le texte est trop long.
-		{
-			// on decoupe le texte en lignes et on limite chaque ligne trop longue.
-			gchar **cLines = g_strsplit (cText, "\n", -1);
-			gchar *cLine;
-			int i;
-			for (i = 0; cLines[i] != NULL; i ++)
-			{
-				cLine = cLines[i];
-				_cairo_dock_limit_string_width (cLine, pLayout, FALSE/*pLabelDescription->bUseMarkup*/, iMaxLineWidth);  // we can't use the markups inside this func, because it works on parts of the string, which can contain piece of markups.
-			}
-			
-			// on reforme le texte et on le passe a pango.
-			gchar *cCutText = g_strjoinv ("\n", cLines);
-			if (pLabelDescription->bUseMarkup)
-				pango_layout_set_markup (pLayout, cCutText, -1);
-			else
-				pango_layout_set_text (pLayout, cCutText, -1);
-			pango_layout_get_pixel_extents (pLayout, NULL, &log);
-			g_strfreev (cLines);
-			g_free (cCutText);
-		}
-	}
-	
-	//\_________________ On cree une surface aux dimensions du texte.
-	gboolean bDrawBackground = (pLabelDescription->fBackgroundColor != NULL && pLabelDescription->fBackgroundColor[3] > 0);
-	double fRadius = fMaxScale * MAX (pLabelDescription->iMargin, MIN (6, pLabelDescription->iSize/4));  // permet d'avoir un rayon meme si on n'a pas de marge.
-	int iOutlineMargin = 2*pLabelDescription->iMargin + (pLabelDescription->bOutlined ? 2 : 0);  // outlined => +1 tout autour des lettres.
+	//\_________________ load the layout into a surface
+	gboolean bDrawBackground = ! pTextDescription->bNoDecorations;
+	double fRadius = (pTextDescription->bUseDefaultColors ? MIN (myStyleParam.iCornerRadius * .75, iSize/2) : fMaxScale * MAX (pTextDescription->iMargin, MIN (6, iSize/2)));  // permet d'avoir un rayon meme si on n'a pas de marge.
+	int iOutlineMargin = 2*pTextDescription->iMargin * fMaxScale + (pTextDescription->bOutlined ? 2 : 0);  // outlined => +1 tout autour des lettres.
 	double fZoomX = ((iMaxWidth != 0 && log.width + iOutlineMargin > iMaxWidth) ? (double)iMaxWidth / (log.width + iOutlineMargin) : 1.);
+	double fLineWidth = 1;
 	
-	*iTextWidth = (log.width + iOutlineMargin) * fZoomX;  // le texte + la marge de chaque cote.
+	*iTextWidth = (log.width + iOutlineMargin) * fZoomX + 2*fLineWidth;  // le texte + la marge de chaque cote.
 	if (bDrawBackground)  // quand on trace le cadre, on evite qu'avec des petits textes genre "1" on obtienne un fond tout rond.
 	{
 		*iTextWidth = MAX (*iTextWidth, 2 * fRadius + 10);
 		if (iMaxWidth != 0 && *iTextWidth > iMaxWidth)
 			*iTextWidth = iMaxWidth;
 	}
-	*iTextHeight = log.height + iOutlineMargin;
+	*iTextHeight = log.height + iOutlineMargin + 2*fLineWidth;
 	
-	cairo_surface_t* pNewSurface = cairo_dock_create_blank_surface (
+	cairo_surface_t* pNewSurface = cairo_dock_create_blank_surface_full (
 		*iTextWidth,
-		*iTextHeight);
+		*iTextHeight,
+		pSourceContext);
 	cairo_t* pCairoContext = cairo_create (pNewSurface);
 	
-	//\_________________ On dessine le fond.
+	//\_________________ draw the background
 	if (bDrawBackground)  // non transparent.
 	{
 		cairo_save (pCairoContext);
-		double fFrameWidth = *iTextWidth - 2 * fRadius;
-		double fFrameHeight = *iTextHeight;
-		cairo_dock_draw_rounded_rectangle (pCairoContext, fRadius, 0., fFrameWidth, fFrameHeight);
-		cairo_set_source_rgba (pCairoContext, pLabelDescription->fBackgroundColor[0], pLabelDescription->fBackgroundColor[1], pLabelDescription->fBackgroundColor[2], pLabelDescription->fBackgroundColor[3]);
-		cairo_fill (pCairoContext);
+		double fFrameWidth = *iTextWidth - 2 * fRadius - fLineWidth;
+		double fFrameHeight = *iTextHeight - fLineWidth;
+		cairo_dock_draw_rounded_rectangle (pCairoContext, fRadius, fLineWidth, fFrameWidth, fFrameHeight);
+		
+		if (pTextDescription->bUseDefaultColors)
+			gldi_style_colors_set_bg_color (pCairoContext);
+		else
+			gldi_color_set_cairo (pCairoContext, &pTextDescription->fBackgroundColor);
+		cairo_fill_preserve (pCairoContext);
+		
+		if (pTextDescription->bUseDefaultColors)
+			gldi_style_colors_set_line_color (pCairoContext);
+		else
+			gldi_color_set_cairo (pCairoContext, &pTextDescription->fLineColor);
+		cairo_set_line_width (pCairoContext, fLineWidth);
+		cairo_stroke (pCairoContext);
+		
 		cairo_restore(pCairoContext);
 	}
 	
@@ -977,7 +916,7 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 		-log.y + dy);
 	
 	//\_________________ On dessine les contours du texte.
-	if (pLabelDescription->bOutlined)
+	if (pTextDescription->bOutlined)
 	{
 		cairo_save (pCairoContext);
 		if (fZoomX != 1)
@@ -996,50 +935,21 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 			pango_cairo_show_layout (pCairoContext, pLayout);
 		}
 		cairo_pop_group_to_source (pCairoContext);
-		//cairo_paint_with_alpha (pCairoContext, .75);
 		cairo_paint (pCairoContext);
 		cairo_restore(pCairoContext);
 	}
 
 	//\_________________ On remplit l'interieur du texte.
-	cairo_pattern_t *pGradationPattern = NULL;
-	if (pLabelDescription->fColorStart != pLabelDescription->fColorStop)
-	{
-		if (pLabelDescription->bVerticalPattern)
-			pGradationPattern = cairo_pattern_create_linear (0.,
-				log.y,
-				0.,
-				log.y + log.height);
-		else
-			pGradationPattern = cairo_pattern_create_linear (log.x,
-				0.,
-				log.x + log.width,
-				0.);
-		g_return_val_if_fail (cairo_pattern_status (pGradationPattern) == CAIRO_STATUS_SUCCESS, NULL);
-		cairo_pattern_set_extend (pGradationPattern, CAIRO_EXTEND_NONE);
-		cairo_pattern_add_color_stop_rgba (pGradationPattern,
-			0.,
-			pLabelDescription->fColorStart[0],
-			pLabelDescription->fColorStart[1],
-			pLabelDescription->fColorStart[2],
-			1.);
-		cairo_pattern_add_color_stop_rgba (pGradationPattern,
-			1.,
-			pLabelDescription->fColorStop[0],
-			pLabelDescription->fColorStop[1],
-			pLabelDescription->fColorStop[2],
-			1.);
-		cairo_set_source (pCairoContext, pGradationPattern);
-	}
+	if (pTextDescription->bUseDefaultColors)
+		gldi_style_colors_set_text_color (pCairoContext);
 	else
-		cairo_set_source_rgb (pCairoContext, pLabelDescription->fColorStart[0], pLabelDescription->fColorStart[1], pLabelDescription->fColorStart[2]);
+		gldi_color_set_cairo_rgb (pCairoContext, &pTextDescription->fColorStart);
 	cairo_move_to (pCairoContext, 0, 0);
 	if (fZoomX != 1)
 		cairo_scale (pCairoContext, fZoomX, 1.);
-	//if (pLabelDescription->bOutlined)
+	//if (pTextDescription->bOutlined)
 	//	cairo_move_to (pCairoContext, 1,1);
 	pango_cairo_show_layout (pCairoContext, pLayout);
-	cairo_pattern_destroy (pGradationPattern);
 	
 	cairo_destroy (pCairoContext);
 	
@@ -1047,6 +957,8 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 	*iTextHeight = *iTextHeight/** / fMaxScale*/;
 	
 	g_object_unref (pLayout);
+	pango_font_description_set_absolute_size (pDesc, iSize * PANGO_SCALE);
+	cairo_destroy (pSourceContext);
 	return pNewSurface;
 }
 
