@@ -18,20 +18,6 @@
 */
 
 #include <math.h>
-#include <sys/time.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h> 
-#include <cairo.h>
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
-
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-#include <X11/Xutil.h>
-#include <gdk/gdkx.h>
-
-#include <GL/glu.h>
 
 #include "cairo-dock-animations.h"  // definition of CairoDockHidingEffect
 #include "cairo-dock-log.h"
@@ -121,9 +107,13 @@ static void _post_render_move_down_opengl (CairoDock *pDock, double fOffset)
 	int iWidth, iHeight;  // taille de la texture
 	iWidth = pDock->container.iWidth;
 	iHeight = pDock->container.iHeight;
-	
+
+	GdkWindow* gdkwindow = gldi_container_get_gdk_window (CAIRO_CONTAINER (pDock));
+	gint scale = gdk_window_get_scale_factor (gdkwindow);
+
 	glPushMatrix ();
 	glLoadIdentity ();
+	glScalef (scale, scale, 1.0);
 	if (!pDock->container.bIsHorizontal)
 	{
 		glTranslatef (iHeight/2, iWidth/2, 0.);
@@ -265,8 +255,12 @@ static void _post_render_fade_out_opengl (CairoDock *pDock, double fOffset)
 			iHeight = pDock->container.iWidth;
 		}
 		
+		GdkWindow* gdkwindow = gldi_container_get_gdk_window (CAIRO_CONTAINER (pDock));
+		gint scale = gdk_window_get_scale_factor (gdkwindow);
+		
 		glPushMatrix ();
 		glLoadIdentity ();
+		glScalef (scale, scale, 1.0);
 		glTranslatef (iWidth/2, iHeight/2, -1.);
 		glScalef (1., -1., 1.);
 		_cairo_dock_apply_texture_at_size_with_alpha (pDock->iRedirectedTexture, iWidth, iHeight, fAlpha);
@@ -327,8 +321,12 @@ static void _post_render_semi_transparent_opengl (CairoDock *pDock, double fOffs
 			iHeight = pDock->container.iWidth;
 		}
 		
+		GdkWindow* gdkwindow = gldi_container_get_gdk_window (CAIRO_CONTAINER (pDock));
+		gint scale = gdk_window_get_scale_factor (gdkwindow);
+		
 		glPushMatrix ();
 		glLoadIdentity ();
+		glScalef (scale, scale, 1.0);
 		glTranslatef (iWidth/2, iHeight/2, -1.);
 		glScalef (1., -1., 1.);
 		_cairo_dock_apply_texture_at_size_with_alpha (pDock->iRedirectedTexture, iWidth, iHeight, fAlpha);
@@ -404,9 +402,13 @@ static void _post_render_zoom_opengl (CairoDock *pDock, double fOffset)
 	_cairo_dock_enable_texture ();
 	_cairo_dock_set_blend_source ();
 	
+	GdkWindow* gdkwindow = gldi_container_get_gdk_window (CAIRO_CONTAINER (pDock));
+	gint scale = gdk_window_get_scale_factor (gdkwindow);
+	
 	double z = _compute_zoom (pDock, fOffset);
 	glPushMatrix ();
 	glLoadIdentity ();
+	glScalef (scale, scale, 1.0);
 	
 	int iWidth, iHeight;  // taille de la texture
 	if (pDock->container.bIsHorizontal)
@@ -499,13 +501,11 @@ static void _post_render_folding_opengl (CairoDock *pDock, double fOffset)
 	iWidth = pDock->container.iWidth;
 	iHeight = pDock->container.iHeight;
 	
-	cairo_dock_set_perspective_view (CAIRO_CONTAINER (pDock));
+	gldi_gl_container_set_perspective_view (CAIRO_CONTAINER (pDock));
 	glPushMatrix ();
 	if (!pDock->container.bIsHorizontal)
 	{
-		//glTranslatef (iHeight/2, iWidth/2, 0.);
 		glRotatef (-90., 0., 0., 1.);
-		//glTranslatef (-iWidth/2, -iHeight/2, 0.);
 		glMatrixMode(GL_TEXTURE);
 		glTranslatef (1./2, 1./2, 0.);
 		glRotatef (-90., 0., 0., 1.);
@@ -514,9 +514,7 @@ static void _post_render_folding_opengl (CairoDock *pDock, double fOffset)
 	}
 	if ((!pDock->container.bDirectionUp && pDock->container.bIsHorizontal) || (pDock->container.bDirectionUp && !pDock->container.bIsHorizontal))
 	{
-		//glTranslatef (0., iHeight/2, 0.);
 		glScalef (1., -1., 1.);
-		//glTranslatef (0., -iHeight/2, 0.);
 		glMatrixMode(GL_TEXTURE);
 		glTranslatef (1./2, 1./2, 0.);
 		glScalef (1., -1., 1.);
@@ -524,9 +522,7 @@ static void _post_render_folding_opengl (CairoDock *pDock, double fOffset)
 		glMatrixMode(GL_MODELVIEW);
 	}
 	
-	//glLoadIdentity ();
-	//glTranslatef (iWidth/2, 0., 0.);
-	glTranslatef (0., -iHeight/2, 0.);
+	glTranslatef (0., -iHeight/2, 0.);  // bottom-middle
 	
 	GLfloat coords[NB_POINTS2*1*8];
 	GLfloat vertices[NB_POINTS2*1*12];
@@ -568,7 +564,7 @@ static void _post_render_folding_opengl (CairoDock *pDock, double fOffset)
 	glEnableClientState (GL_TEXTURE_COORD_ARRAY);
 	glEnableClientState (GL_VERTEX_ARRAY);
 	
-	glScalef (iWidth, iHeight, iHeight/6);
+	glScalef (iWidth, iHeight, iHeight/2);
 	glBindTexture (GL_TEXTURE_2D, pDock->iRedirectedTexture);
 	glTexCoordPointer (2, GL_FLOAT, 2 * sizeof(GLfloat), coords);
 	glVertexPointer (3, GL_FLOAT, 3 * sizeof(GLfloat), vertices);
@@ -577,7 +573,7 @@ static void _post_render_folding_opengl (CairoDock *pDock, double fOffset)
 	glDisableClientState (GL_TEXTURE_COORD_ARRAY);
 	glDisableClientState (GL_VERTEX_ARRAY);
 	
-	cairo_dock_set_ortho_view (CAIRO_CONTAINER (pDock));
+	gldi_gl_container_set_ortho_view (CAIRO_CONTAINER (pDock));
 	glPopMatrix ();
 	
 	if (!pDock->container.bIsHorizontal || !pDock->container.bDirectionUp)
